function [rx,err,errstr]=sendmessage(H,message)

i=0;

% Tipos de erros
OK=0;
ERROR_REQ     =1;
ERROR_SOF     =2;
ERROR_EMPTYMSG=3;
ERROR_INV_RECV=4;
ERROR_INV_SEND=5;
ERROR_BCC_RECV=6;
ERROR_BCC_SEND=7;
ERROR_OPCODE  =8;
ERROR_INVREQ  =9;
ERROR_CPRESET =10;
ERROR_UNKNOWN =-1;

% Load global data
data;

% Validao dos argumentos
if (length(message)~=USART_SEND_LENGTH || (message(1)~=MESSAGE_REQ && message(1)~=MESSAGE_TEST));
    error('Mensagem a transmitir invlida!');
end

% Adicionar byte BCC
message=[message mod(sum(message),256)];

% Inicializao de variveis
rx=[];
err=OK;              % Sem erro no inicio
errstr='OK';

% Mensagem a transmitir
msg=uint8(message);

% A transmisso da mensagem de solicitao  repetida sempre que um erro
% ocorra
tries=0;             % Nmero de tentativas
maxtries=10;         % Nmero mximo de tentativas
err=ERROR_UNKNOWN;
while (err~=OK && tries<=maxtries)
    
    % Reinicializao do estado de Erro
    err=OK;
    
    % Se na segunda tentativa (ou mais), reinicializar os buffers srie
    if (tries>0)
        [status,errstr]=cportreset(H);
        if (status==0)
            err=ERROR_CPRESET;
            break;
        end
    end

    % Contagem do nmero de tentativas
    tries=tries+1;

    % Envio da mensagem de pedido
    n=cportwrite(H,msg);
    if (n>=0)
        msg=msg(n+1:end);
    end

    % Verificando se a transmisso falhou
    if (~isempty(msg))
        err=ERROR_REQ;
        continue;
    end

    % Preparao da mensagem a retransmitir
    msg=uint8(message);

    % Recepo da mensagem de resposta

    % Deteco do primeiro byte da frame
    rx0=0;
    tries2=0;
    maxtries2=20;
    status=0;
    while (status==0 && tries2<=maxtries2)
        [rx0,status]=cportgetint(H,1,1);
        if (status==1 && uint8(intunsigned(rx0,8))<MESSAGE_MINTYPE), status=0; end
        tries2=tries2+1;
    end
    if (status==0)
        err=ERROR_SOF;
        continue
    end

    % Recepo de um array de bytes (bytes seguintes ao primeiro)
    [rx,status]=cportgetint(H,1,USART_RECV_LENGTH);
    rx=[rx0; rx];

    % Se a recepo falhou, voltar a tentar a recepo
    if (status==0)
        err=ERROR_EMPTYMSG;
        continue
    end

    % Neste ponto do programa,  garantido que uma mensagem srie foi
    % obtida

    % Ajuste da mensagem recebida
    rx = uint8(intunsigned(rx,8))';

    % Clculo do BCC da mensagem recebida
    bcc=mod(sum(rx(1:end-1)),256);
    % Comparando o BCC code da mensagem com o ltimo byte
    bcc_err=(bcc~=rx(end));

    % Erro de BCC na mensagem de resposta
    if (bcc_err==1)
        err=ERROR_BCC_RECV;
    % Erro de OpCode no coincidente
    elseif (rx(2)~=message(2))
        err=ERROR_OPCODE;
    % Mensagem transmitida com parmetros invlidos
    elseif (rx(1)==MESSAGE_INVREQ)
        err=ERROR_INVREQ;
    % Mensagem transmitida com erro de BCC
    elseif (rx(1)==MESSAGE_INVALID)
        err=ERROR_BCC_SEND;
    % Master no consegue ler a mensagem de solicitao
    elseif (rx(1)==MESSAGE_ERROR)
        err=ERROR_INV_SEND;
    % Mensagem de natureza desconhecida
    elseif (rx(1)~=MESSAGE_SUCESS && rx(1)~=MESSAGE_TEST)
        err=ERROR_INV_RECV;
    end       
end

% if (tries>1)
%     disp 'XXXXXXXXXXXXXXXXXXXXXX TRIES>1 XXXXXXXXXXXXXXXXXXXX'
%     tries
% end

% Preparao da mensagem de sada
rx=rx(1:end-1);

% Mensagem de erro
if (err~=OK)
    switch err
        case ERROR_REQ
            errstr='Mensagem de solicitao no transmitida!';
        case ERROR_SOF
            errstr='Primeiro byte no detectado!';
        case ERROR_EMPTYMSG
            errstr='Impossivel obter resposta!';
        case ERROR_INV_RECV
            errstr='Mensagem de resposta invlida!';
        case ERROR_INV_SEND
            errstr='Mensagem de solicitao no recebida com sucesso pelo Master!';
        case ERROR_BCC_RECV
            errstr='Mensagem de resposta com erro de BCC!';
        case ERROR_BCC_SEND
            errstr='Mensagem de solicitao com erro de BCC no Master!';
        case ERROR_OPCODE
            errstr='Byte de tipo (OpCode) no coincidente!';
        case ERROR_INVREQ
            errstr='Mensagem com parametros invalidos!';
        case ERROR_CPRESET
            errstr=['Erro na reinicializao dos buffers srie: ',errstr];
        otherwise
            errstr='Erro desconhecido!';
    end
    
    % Reinicializar comunicaes
    cportreset(H);
end