% Estabelecimento de uma nova ligao via RS-232
%
% [handler,error,errorstr]=initcom(gate,rate)
%
% Entradas:
%  gate     -> Porta a utilizar (1,2,...)
%  rate     -> baudrate a definir (bits/s)
% Sadas:
%  handler  -> ID da linha de comunicaes
%  error    -> Cdigo de erro
%  errorstr -> String descritiva do erro
%
% (c) Milton Ruas da Silva, 2006

function [handler,error,errorstr]=initcom(gate,rate)

% Verificao dos argumentos de sada
if (nargout==0)
    error('myfunction:initComm:outargs',' necessrio definir pelo menos uma variavel de sada!');
end

% Argumentos default:
if (nargin<2)
    rate=115200;     % BaudRate=19200
end
if (nargin==0)
    gate=1;         % gate=COM1
end

% Inicializao dos parametros de sada
error=0;
errorstr='OK';

% Abertura da porta de comunicao
handler=cportopen(['com' num2str(gate)]);
if (handler==0)
    error=1;
    errorstr=['Failed to open gate COM',num2str(gate)];
    return;
end

% Configurao do ritmo de transmisso
state=cportconfig(handler,'BaudRate',rate, ...
                          'ByteSize',8, ...
                          'fNull','OFF', ...
                          'fDtrControl','OFF', ...
                          'fRtsControl','OFF', ...
                          'fAbortOnError','ON', ...
                          'ErrorChar',-1, ...
                          'ReadIntervalTimeout',2*11/rate*1000, ...
                          'ReadTotalTimeoutMultiplier',1, ...
                          'ReadTotalTimeoutConstant',10, ...
                          'WriteTotalTimeoutMultiplier',1, ...
                          'WriteTotalTimeoutConstant',10);
                          

% Reset das comunicaes (purging dos buffers de comunicao)
[stat,ErrStr]=cportreset(handler);

% Verificao da ocorrencia de erros no RESET
if (stat==0)
    error=1;
    errorstr='Cannot reset communications!';
    
    % Encerramento da porta
    cportclose(handler);
end

% Apresentao de erros ocorridos antes do RESET
if (~isempty(ErrStr)>0)
    fprintf('Previous errors:\n%s\n',ErrStr);
end
