/*
 *
 * CPORTConfig.C	
 *
 * The calling syntax is:
 *
 *		Status=cportconfig(Port, Configuration)
 *
 */

#include <mex.h>
#include <windows.h>
#include "mxadd.h"                                              /* added because mxArrayToString is not present in Matlab 5.1 */

/* Input Arguments */

#define	PORT_IN	  prhs[0]

/* Output Arguments */

#define	PortState	  plhs[0]

int        number_of_fields=28; 
const char      *field_names[] = {
		"BaudRate",         "ByteSize",   "StopBits",
		"Parity",           "fParity",    "fOutxCtsFlow",
		"fOutxDsrFlow",     "fDtrControl","fDsrSensitivity",
		"fTXContinueOnXoff","fOutX",      "fInX",
		"fErrorChar",       "fNull",      "fRtsControl",
		"fAbortOnError",    "XonLim",     "XoffLim",
		"XonChar",          "XoffChar",   "ErrorChar",
		"EofChar",          "EvtChar",   
		"ReadIntervalTimeout",     "ReadTotalTimeoutMultiplier",
		"ReadTotalTimeoutConstant","WriteTotalTimeoutMultiplier",
		"WriteTotalTimeoutConstant"
		};

enum FieldTypes {BaudRate,         ByteSize,   StopBits,
                 Parity,           fParity,    fOutxCtsFlow,
                 fOutxDsrFlow,     fDtrControl,fDsrSensitivity,
                 fTXContinueOnXoff,fOutX,      fInX,
                 fErrorChar,       fNull,      fRtsControl,
                 fAbortOnError,    XonLim,     XoffLim,
                 XonChar,          XoffChar,   ErrorChar,
                 EofChar,          EvtChar,   
                 ReadIntervalTimeout,     ReadTotalTimeoutMultiplier,
                 ReadTotalTimeoutConstant,WriteTotalTimeoutMultiplier,
                 WriteTotalTimeoutConstant
                };

BOOL StrFits(char *str1, const char *str2)
{
  char c1, c2;
  BOOL ok;

  ok=TRUE;
  while (ok && *str1!='\0' && *str2!='\0')
  {
    c1=*str1; if (c1>='a' && c1<='z') c1=c1-'a'+'A';
    c2=*str2; if (c2>='a' && c2<='z') c2=c2-'a'+'A';
    if (c1 != c2) ok=FALSE;
    str1++; str2++;
  }
  return ok;
}

void Config(DCB		  *PortDCB,
            COMMTIMEOUTS  *CT,
            char          *PName,
            char	  *PValStr,
            double	  Val
	    )
{
  char		*pt;
  DWORD 	DVal;
  int		i;
  int           PType, NFit;
  BOOL 		Exact;

/* Find the property in the list */
  PType=0; NFit=0; Exact=FALSE;
  for (i=0; i<number_of_fields && !Exact; i++)
  {
    if ((strlen(PName) <= strlen(field_names[i])) && (StrFits(PName, field_names[i])))
    {
      PType=i; NFit++;
      if (strlen(PName)==strlen(field_names[i])){
        Exact=TRUE; NFit=1;
      }
    }
  }
  if (NFit < 1)
    mexErrMsgTxt("CPORTCONFIG unrecognized property.");
  if (NFit > 1)
    mexErrMsgTxt("CPORTCONFIG ambiguous property.");

/* Ok, found one and only one fit. */

/* Now, evaluate the property value, from the number or the string */

  if (PValStr != NULL) {
  /* Lowercase */
    for (pt=PValStr; *pt != '\0'; pt++)
      if ((*pt>='A') & (*pt<='Z')) *pt=*pt+'a'-'A';
      
/* Translate */
    if (strcmp(PValStr,"none")==0)          DVal = 0;
    else if (strcmp(PValStr,"odd")==0)      DVal = 1;
    else if (strcmp(PValStr,"even")==0)     DVal = 2;
    else if (strcmp(PValStr,"mark")==0)     DVal = 3;
    else if (strcmp(PValStr,"space")==0)    DVal = 4;
    else if (strcmp(PValStr,"onebit")==0)   DVal = 0;
    else if (strcmp(PValStr,"one5bits")==0) DVal = 1;
    else if (strcmp(PValStr,"twobits")==0)  DVal = 2;
    else if ((strcmp(PValStr,"true")==0) | (strcmp(PValStr,"yes")==0) |
             (strcmp(PValStr,"on")==0))     DVal = 1;
    else if ((strcmp(PValStr,"false")==0) | (strcmp(PValStr,"no")==0) |
             (strcmp(PValStr,"off")==0))    DVal = 0;
    else if ((strcmp(PValStr,"handshake")==0) | (strcmp(PValStr,"handshaking")==0) |
             (strcmp(PValStr,"hand")==0))   DVal = 2;
    else if (strcmp(PValStr,"toggle")==0)   DVal = 3;
    else if (strcmp(PValStr,"-1")==0)       DVal = MAXDWORD;
    else sscanf(PValStr, "%i", &DVal);
  } else {
    if (Val < 0) DVal = MAXDWORD;
    else if (PType==StopBits) {
      if (Val == 1.5)    DVal = 1;
      else if (Val == 2) DVal = 2;
      else               DVal = 0;
    }
    else DVal = Val;
  }

  /* Place port configuration */

  /* First, the port configuration */
  
  if (PType==BaudRate)          (*PortDCB).BaudRate          = DVal;
  if (PType==ByteSize)          (*PortDCB).ByteSize          = DVal;
  if (PType==Parity)            (*PortDCB).Parity            = DVal;
  if (PType==StopBits)          (*PortDCB).StopBits          = DVal;
  if (PType==fParity)           (*PortDCB).fParity           = DVal;
  if (PType==fOutxCtsFlow)      (*PortDCB).fOutxCtsFlow      = DVal;
  if (PType==fOutxDsrFlow)      (*PortDCB).fOutxDsrFlow      = DVal;
  if (PType==fDtrControl)       (*PortDCB).fDtrControl       = DVal;
  if (PType==fDsrSensitivity)   (*PortDCB).fDsrSensitivity   = DVal;
  if (PType==fTXContinueOnXoff) (*PortDCB).fTXContinueOnXoff = DVal;
  if (PType==fOutX)             (*PortDCB).fOutX             = DVal;
  if (PType==fInX)              (*PortDCB).fInX              = DVal;
  if (PType==fErrorChar)        (*PortDCB).fErrorChar        = DVal;
  if (PType==fNull)             (*PortDCB).fNull             = DVal;
  if (PType==fRtsControl)       (*PortDCB).fRtsControl       = DVal;
  if (PType==fAbortOnError)     (*PortDCB).fAbortOnError     = DVal;
  if (PType==XonLim)            (*PortDCB).XonLim            = DVal;
  if (PType==XoffLim)           (*PortDCB).XoffLim           = DVal;
  if (PType==XonChar)           (*PortDCB).XonChar           = DVal;
  if (PType==XoffChar)          (*PortDCB).XoffChar          = DVal;
  if (PType==ErrorChar)         (*PortDCB).ErrorChar         = DVal;
  if (PType==EofChar)           (*PortDCB).EofChar           = DVal;
  if (PType==EvtChar)           (*PortDCB).EvtChar           = DVal;
  
/* Now setup timeouts */
  if (PType==ReadIntervalTimeout)        
     (*CT).ReadIntervalTimeout         = DVal;
  if (PType==ReadTotalTimeoutMultiplier) 
     (*CT).ReadTotalTimeoutMultiplier  = DVal;
  if (PType==ReadTotalTimeoutConstant)   
     (*CT).ReadTotalTimeoutConstant    = DVal;
  if (PType==WriteTotalTimeoutMultiplier)
     (*CT).WriteTotalTimeoutMultiplier = DVal;
  if (PType==WriteTotalTimeoutConstant)  
     (*CT).WriteTotalTimeoutConstant   = DVal;
       
/* mexPrintf("%s = %i\n",PName,DVal);  */

/*
    mexPrintf("ReadIntervalTimeout = %i\n",CT.ReadIntervalTimeout);
    mexPrintf("ReadTotalTimeoutMultiplier = %i\n",CT.ReadTotalTimeoutMultiplier);
    mexPrintf("ReadTotalTimeoutConstant = %i\n",CT.ReadTotalTimeoutConstant);
    mexPrintf("WriteTotalTimeoutMultiplier = %i\n",CT.WriteTotalTimeoutMultiplier);
    mexPrintf("WriteTotalTimeoutConstant = %i\n",CT.WriteTotalTimeoutConstant);
*/

}

void FillNField(
		mxArray *Structure,
		char    *FieldName,
		DWORD  Value
               )
{
  mxArray  *field_value;
  double   *pr;

  field_value = mxCreateDoubleMatrix(1, 1, mxREAL);
  pr = mxGetPr(field_value); 
  if (Value==MAXDWORD)
    *pr = -1;
  else
    *pr = Value;
  mxSetField(Structure, 0, FieldName, field_value);
}

void FillRField(
		mxArray *Structure,
		char    *FieldName,
		double  Value
               )
{
  mxArray  *field_value;
  double   *pr;

  field_value = mxCreateDoubleMatrix(1, 1, mxREAL);
  pr = mxGetPr(field_value); 
  *pr = Value;
  mxSetField(Structure, 0, FieldName, field_value);
}

void FillSField(
		mxArray *Structure,
		char    *FieldName,
		char    *Value
               )
{
  mxArray  *field_value;

  field_value = mxCreateString(Value);
  mxSetField(Structure, 0, FieldName, field_value);
  /*mxDestroyArray(field_value);*/
}

BOOL CPortState(
		   HANDLE hp,
		   mxArray *Structure
		   )
{
  DCB		PortDCB;
  COMMTIMEOUTS  CT;
  BOOL 		ok;
  char    *Tog[]    = {"OFF","ON","HANDSHAKE","TOGGLE"};
  char    *Pty[]    = {"NONE","ODD","EVEN","MARK","SPACE"};
  double  SBits[]   = {1, 1.5, 2};

  ok = TRUE;

  /* Get current port configuration */
  PortDCB.DCBlength = sizeof(DCB);
  if (ok & !GetCommState(hp, &PortDCB)) ok = FALSE;
  if (ok & !GetCommTimeouts(hp,&CT)) ok = FALSE;
 
  /* Populate the configuration structure */
  if (ok) {
    FillNField(Structure,"BaudRate",         PortDCB.BaudRate);
    FillNField(Structure,"ByteSize",         PortDCB.ByteSize);
    FillSField(Structure,"Parity",           Pty[PortDCB.Parity]);
    FillRField(Structure,"StopBits",         SBits[PortDCB.StopBits]);
    FillSField(Structure,"fParity",          Tog[PortDCB.fParity]);
    FillSField(Structure,"fOutxCtsFlow",     Tog[PortDCB.fOutxCtsFlow]);
    FillSField(Structure,"fOutxDsrFlow",     Tog[PortDCB.fOutxDsrFlow]);
    FillSField(Structure,"fDtrControl",      Tog[PortDCB.fDtrControl]);
    FillSField(Structure,"fDsrSensitivity",  Tog[PortDCB.fDsrSensitivity]);
    FillSField(Structure,"fTXContinueOnXoff",Tog[PortDCB.fTXContinueOnXoff]);
    FillSField(Structure,"fOutX",            Tog[PortDCB.fOutX]);
    FillSField(Structure,"fInX",             Tog[PortDCB.fInX]);
    FillSField(Structure,"fErrorChar",       Tog[PortDCB.fErrorChar]);
    FillSField(Structure,"fNull",            Tog[PortDCB.fNull]);
    FillSField(Structure,"fRtsControl",      Tog[PortDCB.fRtsControl]);
    FillSField(Structure,"fAbortOnError",    Tog[PortDCB.fAbortOnError]);
    FillNField(Structure,"XonLim",           PortDCB.XonLim);
    FillNField(Structure,"XoffLim",          PortDCB.XoffLim);
    FillNField(Structure,"XonChar",          PortDCB.XonChar);
    FillNField(Structure,"XoffChar",         PortDCB.XoffChar);
    FillNField(Structure,"ErrorChar",        PortDCB.ErrorChar);
    FillNField(Structure,"EofChar",          PortDCB.EofChar);
    FillNField(Structure,"EvtChar",          PortDCB.EvtChar);

    FillNField(Structure,"ReadIntervalTimeout",        CT.ReadIntervalTimeout);
    FillNField(Structure,"ReadTotalTimeoutMultiplier", CT.ReadTotalTimeoutMultiplier);
    FillNField(Structure,"ReadTotalTimeoutConstant",   CT.ReadTotalTimeoutConstant);
    FillNField(Structure,"WriteTotalTimeoutMultiplier",CT.WriteTotalTimeoutMultiplier);
    FillNField(Structure,"WriteTotalTimeoutConstant",  CT.WriteTotalTimeoutConstant);

    return TRUE;
  } else 
    return FALSE;

}


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  HANDLE	      *hp;
  double	      *ParamVal, Val;
  char		      *ParamStr, *ParamName;
  DCB 		      PortDCB;
  COMMTIMEOUTS	CT;
  BOOL	      	ok;
  INT		        i;
  
  mxArray    *Structure;
  int        ndim = 2, dims[2] = {1, 1};


  /* Check for proper number of arguments */

  i = nrhs%2;
  if (2*i == nrhs )
    mexErrMsgTxt("CPORTCONFIG incorrect number of input arguments.");
  if (nlhs > 1)
    mexErrMsgTxt("CPORTCONFIG too many output arguments.");
  
  /* Check the type of PORT_IN - should be double */
  if (!mxIsDouble(PORT_IN))
    mexErrMsgTxt("CPORTCONFIG first argument should be a port handle.");


  /* Assign pointers to the port handle */

  hp   = (HANDLE *)mxGetData(PORT_IN);


/****************************/
/* Do the actual operations */
/****************************/

 
/* Change configuration? */

  ParamName = NULL; ParamStr = NULL; Val = 0;
  
  if (nrhs > 1) {
    ok = TRUE;
/* Get current port configuration */

    PortDCB.DCBlength = sizeof(DCB);
    if (!GetCommState(*hp, &PortDCB))   ok = FALSE;
    if (ok & !GetCommTimeouts(*hp,&CT)) ok = FALSE;
    
/* Get changes */
    for (i=1; ok & (i<nrhs); i=i+2) {
      ok = mxIsChar(prhs[i]);
      if (ok) {
        ParamName = mxArrayToString(prhs[i]);
        if (mxIsDouble(prhs[i+1])) {
          ParamVal = (double *)mxGetPr(prhs[i+1]);
          Val = *ParamVal; ParamStr = NULL;
        } else if (mxIsChar(prhs[i+1])) {
          ParamStr = mxArrayToString(prhs[i+1]);
          Val = 0;
        }
        Config(&PortDCB, &CT, ParamName, ParamStr, Val); 
        mxFree(ParamName);
        if (ParamStr != NULL) mxFree(ParamStr);
        ParamName = NULL; ParamStr = NULL; Val = 0;
      } else
        mexErrMsgTxt("CPORTCONFIG Illegal property name.");
    }
    if (ok) {
      if (!SetCommState(*hp, &PortDCB)) ok = FALSE; 
      if (!SetCommTimeouts(*hp, &CT))   ok = FALSE;
    }
  }

/*******************************************/

/* Return the configuration state? */

  if (nlhs > 0 ) {
/* Create a matrix for the return argument and assign pointers */

    Structure = mxCreateStructArray(ndim, dims, 
                                    number_of_fields, field_names);
                                    
    if (CPortState(*hp, Structure))
      PortState = Structure;
    else {
      PortState = mxCreateDoubleMatrix(0,0,mxREAL);
      mxDestroyArray(Structure);
    }
  }
  
  return;
}


