/*
 *
 * CPORTPutInt.C
 *
 * The calling syntax is:
 *
 *      nBytesWritten=cportputint(H,X,nbyte)
 */

#include <mex.h>
#include <windows.h>

/* Input Arguments */

#define	PORT_IN	 prhs[0]
#define DATA_IN  prhs[1]
#define NBYTE_IN prhs[2]

/* Output Arguments */

#define	NB_OUT   plhs[0]


long num2longtsr(
                 const double x,
                 const int upper,
                 const int lower
                )
/*
 *
 * num2longtsr
 *
 *  Saturates a value if it goes above an upper bound or below a lower bound.
 *  Rounds to nearest long integer value.
 *
 */
{
    if (x < lower)
        return lower;
    else if (x > upper)
        return upper;
    else 
        return (long)(x<0 ? x - 0.5 : x + 0.5);

}


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		        )
{
    HANDLE	*hp;
    double	*Pr, *Pi;
    char    *ch;
    BOOL    ok, complex;
    long    word, MaxWord, MinWord; 
    DWORD   nBytes, nCharToWrite, nCharWritten, nToWrite, word_ind, byte_ind, ch_ind;

    /* Check for proper number of arguments */

    if ((nrhs < 1) | (nrhs > 3))
        mexErrMsgTxt( "CPORTGETINT requires 2 to 3 input arguments." );
    if (nlhs > 1) 
        mexErrMsgTxt( "CPORTGETINT requires at most 1  output argument." );
  
    /* Check the type of PORT_IN and assign pointer */
    if (!mxIsDouble(PORT_IN))
        mexErrMsgTxt("CPORTGETINT first argument should be a file handle.");
    hp = (HANDLE *)mxGetData( PORT_IN );

    if (!mxIsDouble( DATA_IN ))
        mexErrMsgTxt( "CPORTPUTINT second argument should be a number." );

    /* if nbyte specified */
    if (nrhs > 2)
    {
        if (!mxIsDouble( NBYTE_IN ))
            mexErrMsgTxt( "CPORTPUTINT third argument should be a number." );
        Pr = mxGetPr( NBYTE_IN );
        nBytes = num2longtsr( *Pr, sizeof( long ), 1  );
    }
    else
        nBytes = 2;


    /* Create a matrix for the return arguments */
    NB_OUT = mxCreateDoubleMatrix( 1, 1, mxREAL );

    nToWrite    = mxGetNumberOfElements( DATA_IN );
    complex = (mxIsComplex( DATA_IN ) != 0);
    if (complex)
        nCharToWrite    = 2*nToWrite*nBytes;
    else
        nCharToWrite    = nToWrite*nBytes;

    ch = mxMalloc( nCharToWrite );

    MaxWord = 0x7F;
    for (byte_ind = 1; byte_ind < nBytes; byte_ind++)
    {
        MaxWord <<= 8;
        MaxWord |= 0xFF;
    }
    MinWord = -MaxWord - 1;

    Pr  = mxGetPr( DATA_IN );
    if (complex)
    {
        Pi  = mxGetPi( DATA_IN );
        for (word_ind = 0, ch_ind   = 0; word_ind < nToWrite; word_ind++)
        {
            word    = num2longtsr( Pr[word_ind], MaxWord, MinWord  );
            for (byte_ind = 0; byte_ind < nBytes; byte_ind++, ch_ind++)
            {
                ch[ch_ind] = (char)word;
                word >>= 8;
            }

            word    = num2longtsr( Pi[word_ind], MaxWord, MinWord );
            for (byte_ind = 0; byte_ind < nBytes; byte_ind++, ch_ind++)
            {
                ch[ch_ind] = (char)word;
                word >>= 8;
            }
       }
    }
    else
    {
        for (word_ind = 0, ch_ind   = 0; word_ind < nToWrite; word_ind++)
        {
            word    = num2longtsr( Pr[word_ind], MaxWord, MinWord );
            for (byte_ind = 0; byte_ind < nBytes; byte_ind++, ch_ind++)
            {
                ch[ch_ind] = (char)word;
                word >>= 8;
            }
       }
    }

    ok = (*hp != 0);
    nCharWritten = 0;

    /* Do it */
    if (ok & !WriteFile(*hp, ch, nCharToWrite, &nCharWritten, NULL))
        ok = FALSE; 

    /* output */
    Pr  = mxGetPr( NB_OUT );
    Pr[0]   = nCharWritten;

    mxFree( ch );
    
    return;
}

