/*
 *
 * CPORTReset.C	
 *
 * The calling syntax is:
 *
 *		[rc,ErrStr]=cportreset(H)
 *
 */

#include <mex.h>
#include <windows.h>

/* Input Arguments */

#define	PORT_IN	prhs[0]

/* Output Arguments */

#define	STATUS	  plhs[0]
#define STAT_STR  plhs[1];


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  HANDLE	*hp;
  double	*Stat;
  DWORD		ErrStat;
  BOOL		ok;

  /* Check for proper number of arguments */
  
  if (nrhs != 1) {
    mexErrMsgTxt("CPORTRESET requires one input argument.");
  }
  if (nlhs > 2) {
    mexErrMsgTxt("CPORTRESET requires up to two output argument2.");
  }
  
  /* Check the type of PORT_IN - should be double */

  if (!mxIsDouble(PORT_IN)) {
    mexErrMsgTxt("CPORTRESET first argument should be a file handle.");
  }

  /* Assign pointers to the various parameters */

  hp = (HANDLE *)mxGetData(PORT_IN);
  
  /* Do the actual operation */
  
  ok = TRUE; ErrStat = 0;
  if (ClearCommError(*hp, &ErrStat, NULL)) {  
    if (!PurgeComm(*hp, PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR)) 
      ok = FALSE;
  } else ok = FALSE;
  
/* Return */
  if (nlhs > 0) {
    STATUS = mxCreateDoubleMatrix(1, 1, mxREAL);
    Stat  = (double *)mxGetPr(STATUS);
    if (ok) *Stat = 1; else *Stat = 0;
  }

  if (nlhs > 1) {
    if ((ErrStat & CE_MODE)!=0)
        plhs[1] = mxCreateString("Invalid port handle or unsupported mode");
    else if ((ErrStat & CE_BREAK)!=0) 
        plhs[1] = mxCreateString("The hardware detected a break condition");
    else if ((ErrStat & CE_DNS)!=0) 
        plhs[1] = mxCreateString("A parallel device is not selected");
    else if ((ErrStat & CE_FRAME)!=0) 
        plhs[1] = mxCreateString("The hardware detected a framing error");
    else if ((ErrStat & CE_IOE)!=0) 
        plhs[1] = mxCreateString("An I/O error occurred during communication with the device");
    else if ((ErrStat & CE_OOP)!=0) 
        plhs[1] = mxCreateString("A parallel device is out of paper");
    else if ((ErrStat & CE_OVERRUN)!=0) 
        plhs[1] = mxCreateString("A character buffer overrun has occurred. The next character is lost");
    else if ((ErrStat & CE_PTO)!=0) 
        plhs[1] = mxCreateString("A parallel device has signaled a time-out");
    else if ((ErrStat & CE_RXOVER)!=0) 
        plhs[1] = mxCreateString("Input buffer overflow");
    else if ((ErrStat & CE_RXPARITY)!=0) 
        plhs[1] = mxCreateString("Parity error");
    else if ((ErrStat & CE_TXFULL)!=0) 
        plhs[1] = mxCreateString("The output buffer is full");
    else
        plhs[1] = mxCreateString("");
  }
  return;
}


