/*
 *
 * CPORTState.C	
 *
 * The calling syntax is:
 *
 *		State=cportOpen(hPort)
 *
 */

#include <mex.h>
#include <windows.h>

/* Input Arguments */

#define	PORT_IN	prhs[0]

/* Output Arguments */

#define PortState plhs[0]

BOOL CPortState(
		   HANDLE hp,
		   DCB *PortDCB,
		   COMMTIMEOUTS *CT
		   )
{
  BOOL ok;

  ok = TRUE;

  /* Get current port configuration */
  (*PortDCB).DCBlength = sizeof(DCB);
  if (ok & !GetCommState(hp, PortDCB)) ok = FALSE;
  if (ok & !GetCommTimeouts(hp,CT)) ok = FALSE;
 
  if (ok)
    return TRUE;
  else 
    return FALSE;

}

void FillField(
		mxArray *Structure,
		char    *FieldName,
		DWORD  Value
               )
{
  mxArray  *field_value;
  double   *pr;

  field_value = mxCreateDoubleMatrix(1, 1, mxREAL);
  pr = mxGetPr(field_value); 
  if (Value==MAXDWORD)
    *pr = -1;
  else
    *pr = Value;
  mxSetField(Structure, 0, FieldName, field_value);
  /*mxDestroyArray(field_value);*/
}


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  HANDLE	*hp;
  DCB		PortDCB;
  COMMTIMEOUTS  CT;
  mxArray       *Structure;

  int        ndim = 2, dims[2] = {1, 1};
  int        number_of_fields=21; 
  const char *field_names[] = {
        "BaudRate",         "ByteSize",   "StopBits",
        "Parity",           "fParity",    "fOutxCtsFlow",
        "fOutxDsrFlow",     "fDtrControl","fDsrSensitivity",
        "fTXContinueOnXoff","fOutX",      "fInX",
        "fErrorChar",       "fNull",      "fRtsControl",
        "fAbortOnError",       
        
        "ReadIntervalTimeout",     "ReadTotalTimeoutMultiplier",
        "ReadTotalTimeoutConstant","WriteTotalTimeoutMultiplier",
        "WriteTotalTimeoutConstant"
  };

  /* Check for proper number of arguments */
  
  if (nrhs != 1) {
    mexErrMsgTxt("CPORTOPEN requires one input argument.");
  }
  if (nlhs != 1) {
    mexErrMsgTxt("CPORTOPEN requires one output argument.");
  }
  
  /* Check the type of PORT_IN - should be double */

  if (!mxIsDouble(PORT_IN)) {
    mexErrMsgTxt("CPORTOPEN first argument should be a port handle.");
  }

  /* Allocate and translate the input parameter string */

  hp   = (HANDLE *)mxGetData(PORT_IN);
  
  /* Create a matrix for the return argument and assign pointers */

  Structure = mxCreateStructArray(ndim, dims, 
                                  number_of_fields, field_names);
 
  /* Do the actual operations in a subroutine */
  
  if (CPortState(*hp, &PortDCB, &CT)) {

  /* Populate the configuration structure */

    FillField(Structure,"BaudRate",         PortDCB.BaudRate);
    FillField(Structure,"ByteSize",         PortDCB.ByteSize);
    FillField(Structure,"Parity",           PortDCB.Parity);
    FillField(Structure,"StopBits",         PortDCB.StopBits);
    FillField(Structure,"fParity",          PortDCB.fParity);
    FillField(Structure,"fOutxCtsFlow",     PortDCB.fOutxCtsFlow);
    FillField(Structure,"fOutxDsrFlow",     PortDCB.fOutxDsrFlow);
    FillField(Structure,"fDtrControl",      PortDCB.fDtrControl);
    FillField(Structure,"fDsrSensitivity",  PortDCB.fDsrSensitivity);
    FillField(Structure,"fTXContinueOnXoff",PortDCB.fTXContinueOnXoff);
    FillField(Structure,"fOutX",            PortDCB.fOutX);
    FillField(Structure,"fInX",             PortDCB.fInX);
    FillField(Structure,"fErrorChar",       PortDCB.fErrorChar);
    FillField(Structure,"fNull",            PortDCB.fNull);
    FillField(Structure,"fRtsControl",      PortDCB.fRtsControl);
    FillField(Structure,"fAbortOnError",    PortDCB.fAbortOnError);

    FillField(Structure,"ReadIntervalTimeout",        CT.ReadIntervalTimeout);
    FillField(Structure,"ReadTotalTimeoutMultiplier", CT.ReadTotalTimeoutMultiplier);
    FillField(Structure,"ReadTotalTimeoutConstant",   CT.ReadTotalTimeoutConstant);
    FillField(Structure,"WriteTotalTimeoutMultiplier",CT.WriteTotalTimeoutMultiplier);
    FillField(Structure,"WriteTotalTimeoutConstant",  CT.WriteTotalTimeoutConstant);

    PortState = Structure;
  } else PortState = NULL;

  return;
}


