/*
 * CPORTWrite.C
 *
 * The calling syntax is:
 *
 *		nWritten=cportWrite(Port,Data,EOL)
 *
 */

#include <mex.h>
#include <windows.h>
#include "mxadd.h"                                              /* added because mxArrayToString is not present in Matlab 5.1 */

/* Input Arguments */

#define	PORT_IN	prhs[0]
#define DATA_IN prhs[1]
#define EOL_IN  prhs[2]

/* Output Arguments */

#define	NW	  plhs[0]


BOOL CPortWrite(
		   HANDLE *hp,
                   void *Data,
	  	   DWORD nWrite,
		   DWORD *nWritten,
		   char  *EOL
		   )
{
  char   EOLStr[3];
  DWORD  nWEOL;

  if (!WriteFile(*hp, Data, nWrite, nWritten, NULL)) return FALSE;
  /* mexPrintf("%s",Data); */
  if (EOL != NULL) {
    EOLStr[0] = '\0'; EOLStr[1] = '\0'; EOLStr[2] = '\0';
    if ((strcmp(EOL,"cr")==0)   || (strcmp(EOL,"CR")==0  )) EOLStr[0] = 13;
    if ((strcmp(EOL,"lf")==0)   || (strcmp(EOL,"LF")==0  )) EOLStr[0] = 10;
    if ((strcmp(EOL,"crlf")==0) || (strcmp(EOL,"CRLF")==0)) {
      EOLStr[0] = 13; EOLStr[1] = 10;
    }
    nWrite = strlen(EOLStr);
    if (!WriteFile(*hp, EOLStr, nWrite, &nWEOL, NULL)) return FALSE;
    *nWritten += nWEOL;
    /* mexPrintf("%s",EOLStr); */
  }
  return TRUE;
}

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  void		*Data;
  HANDLE	*hp;
  DWORD		nWritten, nWrite;
  double	*dnWritten, nW;
  char		*EOL;

  /* Check for proper number of arguments */
  
  if (nrhs < 2) {
    mexErrMsgTxt("CPORTWRITE requires at least two input arguments.");
  }
  if (nrhs > 3) {
    mexErrMsgTxt("CPORTWRITE requires at most three input arguments.");
  }
  if (nlhs > 1) {
    mexErrMsgTxt("CPORTWRITE requires one output argument at most.");
  }
  
  /* Check the type of PORT_IN - should be double */

  if (!mxIsDouble(PORT_IN)) {
    mexErrMsgTxt("CPORTWRITE first argument should be a port handle.");
  }

  /* Allocate and translate the input data string */

  if (mxIsChar(DATA_IN)) {
    Data   = (void *)mxArrayToString(DATA_IN);
    nWrite = mxGetNumberOfElements(DATA_IN);
  } else {
    Data   = mxGetData(DATA_IN);
    nWrite = mxGetNumberOfElements(DATA_IN) * mxGetElementSize(DATA_IN);
  }
  nWritten = 0;

  /* EOL */
  EOL=NULL;
  if (nrhs==3) {
    if (!mxIsChar(EOL_IN)) 
      mexErrMsgTxt("CPORTWRITE third argument should be a string.");
    EOL=mxArrayToString(EOL_IN);
  }
  
  /* Assign pointers to the various parameters */

  hp = (HANDLE *)mxGetData(PORT_IN);
 
  /* Do the actual computations in a subroutine */
  
  if (CPortWrite(hp, Data, nWrite, &nWritten, EOL)) 
    nW = nWritten;
  else
    nW = -1;

  if (nlhs > 0) {
    NW = mxCreateDoubleMatrix(1, 1, mxREAL);
    dnWritten  = (double *)mxGetPr(NW);
    *dnWritten = nW;
  }
  return;
}
