/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: MasterSec.h                                             |
|                                                                        |
|                            HEADER FILE                                 |
+=======================================================================*/


#ifndef _USART_H
#define _USART_H

#include "global.h"

// Byte inicial das mensagens USART
#define MESSAGE_REQ     (0xFF)            // Start Of Frame utilizado numa mensagem de request (PC->PIC)
#define MESSAGE_TEST    (0xFE)            // SOF utilizado para a sequencia de teste
#define MESSAGE_SUCESS  (0xFD)            // SOF utilizado em caso de sucesso
#define MESSAGE_INVREQ  (0xFC)            // Pedido com parmetros invlidos
#define MESSAGE_INVALID (0xFB)            // Mensagem de estrutura invlida
#define MESSAGE_ERROR   (0xFA)            // Mensagem de erro

// Inicializacao da USART
extern void usartInit(void);

// Inicializao da contagem de tempo do timer0
extern void usartCountTimeStart(void);

// Verificao de ocorrencia de timeout
extern bool usartCountTimeout(void);

// Armazenamento de uma mensagem no buffer externo da USART.
// Este buffer  necessrio para a transmisso uma mensagem
// configurvel pelo utilizador, seguindo a poltica Request->Send!
extern void usartStore2extBuff(byte *message, byte n);

// _____ FUNES PARA PREPARAO DE MENSAGENS DE RESPOSTA ________________

// Processamento de uma mensagem recebida pela Usart (a partir do PC)
extern void usartProcessMsg(void);

//*****************************************************************************
// Envio de uma mensagem de status
//
extern void usartSendStatus(byte stat);

// _____ FUNES DE MANIPULAO DIRECTA DO BUFFER RX/TX __________________

// Retorna true, se o buffer de recepo est a receber um stream de bytes
extern bool usartStreamMode(void);

// Insero de um byte no buffer de recepo.
// Retorna true, se a insero teve sucesso
// *full indica se o buffer de recepo est cheio
extern bool usartStoreRx(byte val, bool *full);

// Obteno e remoo de um byte do buffer de transmisso.
// *var conter o byte removido;
// *empty indica se o buffer de transmisso est vazio
// True  retornado se a obteno teve sucesso.
extern bool usartGetTx(byte *var, bool *empty);

// Reset do Buffer de recepo.
// O buffer  esvaziado e pronto para a recepo de novos caracteres.
extern void usartResetRxBuff(void);

// Reset do Buffer de transmisso.
// O buffer  esvaziado e pronto para armazenar novos bytes.
extern void usartResetTxBuff(void);

#endif
