#include "servo.h"
#include "pic2.h"
#include "can.h"
#include "data.h"
#include <string.h>

struct_sensorHighLevel sensorHighLevel;

// Estrutura com os dados do controlador
typedef struct {
    double refInput;            // Entrada referncia do controlador
    double angle;               // ngulo a solicitar
    signed int  error_prev;     // Erro do ciclo anterior
    signed char y_prev;         // Sinal de feedback (potencimetro) do ciclo anterior
    signed char y_prev2;        // Sinal de feedback de h 2 ciclos atrs
} struct_controller;

// Estrutura com os dados do controlador
typedef struct  {
    byte type;                      // Tipo de controlador em aco
    bool errortol;                    // Deve tolerar pequenos erros?
    signed char refPosition;      // Posio final a atingir
    struct_controller controller;   // Dados do controlador
} struct_control;
static struct_control servoControl[N_SERVOS];      // Dados dos actuadores

// Estrutura utilizada para a calibrao dinmica (dynamicServoCalib function)
struct {
    signed char pos0;           // Posio original
    signed int deltamax;        // Mximo deslocamento
} dynCalib[N_SERVOS];

// Execuo da calibrao dos servos
//
void calibration(void)
{
    byte state, i, count;
    signed int med;
    unsigned int special_sum[N_SPECIAL_SENSORS];
    signed int position_sum[N_SERVOS];
    signed char posReq[N_SERVOS];

    // If going to calculate the multiplier factor, wait 3 seconds
    wait(1000);

    // ___ CALIBRAO DOS SENSORES DE FORA ____________________________
    // Inicializao das variveis locais
    for (i=0; i<N_SPECIAL_SENSORS; ++i)  {
        special0[i].offset=0;
        special0[i].slope=100;
    }

    // Offset and slope correction of the pressure sensors
    // In this situation, only the offset calibration is done (state<=1)
    for (state=1; state<=1; ++state)  {

        // Clearing the sum array
        for (i=0; i<N_SPECIAL_SENSORS; ++i)  special_sum[i]=0;

        // Execuo de 25 medidas
        for (count=0; count<25; ++count)  {
    		// Esperando pela passagem de um tick;
            waitTick();
    
            // Acumulao da sada dos sensores de fora
            for (i=0; i<N_SPECIAL_SENSORS; ++i)
                special_sum[i] += sensors.special[i];
        }
        // Correco dos desvios dos sensores de fora
        for (i=0; i<N_SPECIAL_SENSORS; ++i)  {
            // Medium read value
            med = (signed int) (special_sum[i]/25);

            switch (state)  {
            // Offset determinado
            case 1: special0[i].offset = (signed int) 128 - med;
                    break;
            // Calculing Slope
            // After the 3 seconds the foot must be located over the ground!
            case 2: special0[i].slope = (signed int) 128*100/(signed int) med;
                    break;
            }
        }
    }

    // ___ CALIBRAO DOS POTENCIMETROS DOS SERVOS ____________________
    // Inicializao de variveis relativas aos servos e activao do PWM
    for (i=0; i<N_SERVOS; ++i)  {

        // Inicializao da estrutura para calibrao dinmica (dynamicServoCalib function)
        dynCalib[i].pos0=posReq[i];
        dynCalib[i].deltamax=0;

        // Inicializao do array
        position_sum[i]=0;

        // Posio solicitada
        posReq[i]=servo_interface[i].actuation;

        // Activao do sinal de PWM
        statusPWM(true,i);

        // Wait some time
        wait(500);
    }

    // Execuo de 25 medidas
    for (count=0; count<25; ++count)  {
        // Esperando pela passagem de um tick;
        waitTick();

        // Acumulao das posies medidas
        for (i=0; i<N_SERVOS; ++i)
            position_sum[i] += servo_interface[i].position;
    }

    // Correco dos desvios da medio do potencimetro
    for (i=0; i<N_SERVOS; ++i)  {
        med = (signed int) (position_sum[i]/25);
        origin[i].offset += (signed int) posReq[i]-med;
    }
    // Calibrao efectuada!
}

// Calibrao do declive para clculo da posio angular
bool dynamicServoCalib(signed char posact, byte srv)
{
    signed int  delta;
    signed long offsetnew, slopenew;
    signed char pos0=dynCalib[srv].pos0;                    // Posio original (da ltima calibrao esttica)

    unsigned char str[]="\r\nNova Calibracao: Servo=";
    unsigned char strslope[]="     Slope=";
    unsigned char stroffset[]="  Offset=";

    // Verificando se est autorizada a calibrao dinmica
    if (!(sensors.sysStatus.calib=actuators.sysStatus.calib))  return false;

    // Apenas  feita a calibrao dinmica, se o controlo est em malha aberta.
    if (actuators.control[srv].type!=NO_CONTROL)  return false;

    // Diferena do deslocamento relativamente  posio original (mdulo)
    if ((delta=(signed int) pos0-(signed int) posact)<0)  delta*=-1;

    // Apenas faz calibrao dinmica se o deslocamento  superior ou igual a 10 e se
    // a diferena angular desde a posio original for mxima
    if (delta<10 || delta<=dynCalib[srv].deltamax)  return false;

    // Actualizao do mximo deslocamento
    dynCalib[srv].deltamax=delta;

    // Novo declive
    slopenew=(signed long) origin[srv].slope*((signed long) pos0-(signed long) posact)/((signed long) pos0-(signed long) servo_interface[srv].position);

    // Novo offset
    offsetnew=slopenew*((signed long) origin[srv].offset-(signed long) pos0)/(signed long) origin[srv].slope+(signed long) pos0;

    // Actualizao do declive e offset
    origin[srv].slope =(signed int) slopenew;
    origin[srv].offset=(signed int) offsetnew;
/*
    // Apresentao dos resultados finais
    sendStr(str);
    sendVal(srv,10);
    sendStr(strslope);
    sendValSign(origin[srv].slope,10);
    sendStr(stroffset);
    sendValSign(origin[srv].offset,10);
*/
    return true;
}


// Movimentao do servo para a posio solicitada
//
signed char servoRequest(signed char position, byte control_type, byte srv)
{
    // Correco da posio solicitada
    position=limitPosition(position);

    // Posio final a atingir
    servoControl[srv].refPosition=position;

    // Tipo de controlador a aplicar
    servoControl[srv].type=control_type;
    
    // Tolerncia a erros
    servoControl[srv].errortol=true;       // Tolaerancia a pequenos erros de posio

    return position;
}

// ==========================================================================================
// Funes temporais
// ==========================================================================================

// Inicializao do Controlador
//
void initController(void)
{
    byte srv;
    struct_controller *cont;

    // Inicializao da estrutura com os dados sensoriais de alto-nvel
    memset(&sensorHighLevel,0,sizeof(struct_sensorHighLevel));

    for (srv=0; srv<N_SERVOS; ++srv)  {

        cont = &(servoControl[srv].controller);

        servoControl[srv].type=NO_CONTROL;
        servoControl[srv].errortol=true;
        servoControl[srv].refPosition=servo_interface[srv].actuation;
        cont->refInput=(double) servoControl[srv].refPosition;
        cont->angle=cont->refInput;
        cont->error_prev=0;
        cont->y_prev=0;
        cont->y_prev2=0;
    }
}

// Controladores de primeiro nvel
static void openLoopControl(byte servo);
static void inclinControl(byte servo);
static void jacobianControl(byte servo, signed int Jt[N_SERVOS][N_SERVOS]);

// Controlador local
static void localControl(byte servo);

void controller(void)
{
    byte servo;
    signed int Jt[N_SERVOS][N_SERVOS];

    // Clculo da matriz Jacobiana
    jacobianMatrix(Jt);

    // Clculo do Centro de presso
    calcCoP(sensorHighLevel.cop);
    // Clculo da inclinao
    calcInc(sensorHighLevel.inc);

    for (servo=0; servo<N_SERVOS; ++servo)  {

        sendValSign(servo_interface[servo].actuation,10);
        sendChar(' ');

        // Passagem da posio do servomotor
        sensorHighLevel.servo[servo]=(signed int) servo_interface[servo].position;

        // Aplicao do algoritmo de controlo de primeiro nvel
        // (O tipo de controlador foi definido no ltima chamada a servoRequest)
        switch (servoControl[servo].type)  {
            // Controlo do Centro de Presso
            case COP_CONTROL:   jacobianControl(servo,Jt);
                                break;
            // Controlo da inclinao
            case INC_CONTROL:   inclinControl(servo);
                                break;
            // Sem controlo de primeiro nvel
            default:            openLoopControl(servo);
                                break;

        }

        // Controlo local
        localControl(servo);
    }

    sendChar('\r');
    sendChar('\n');
}


// _____ SEM CONTROLADOR ______________________________________________________________

static void openLoopControl(byte servo)
{
    servoControl[servo].controller.refInput=(double) servoControl[servo].refPosition;
}

// _____ CONTROLADOR DE INCLINAO _______________________________________
// Processamento e aplicao da nova velocidade a aplicar
static void applyVelocity(byte servo, double velocity);

#define KP_FORCEY    0.1
#define KP_FORCEX    0.05

static void inclinControl(byte servo)
{
    double delta;
	double K = (double) actuators.control[servo].kd;

    if (servo==0)  {
        // COMPENSAO DO CENTRO DE PRESSO NO EIXO XX
        // Clculo da variao de posio a aplicar
        delta = (double) servo_signal[servo]*sensorHighLevel.inc[0]*2.5*K*1e-2;
    
        // Aplicao da nova velocidade ao servo 0
        applyVelocity(servo,delta);
    }
    else if (servo==1)  {
        // COMPENSAO DO CENTRO DE PRESSO NO EIXO YY
        // Actualizao da posio do servo
        delta = (double) servo_signal[servo]*sensorHighLevel.inc[1]*2.5*K*1e-2;
    
        // Aplicao da nova velocidade ao servo 1
        applyVelocity(servo,delta);
    }
}

// _____ CONTROLADOR DO COP PELO JACOBIANO _____________________________________

static const signed char K_error[3]={10, 10, -20};      // Valor normalizado==10

static void jacobianControl(byte servo, signed int Jt[N_SERVOS][N_SERVOS])
{
    double delta;
    double K=(double) actuators.control[servo].kd;
    signed int copx_error, copy_error, copz_error;

    copx_error=((signed int) servoControl[0].refPosition-sensorHighLevel.cop[0])*K_error[0]/(signed int) 10;
    copy_error=((signed int) servoControl[1].refPosition-sensorHighLevel.cop[1])*K_error[1]/(signed int) 10;
    copz_error=((signed int) servoControl[2].refPosition-sensorHighLevel.cop[2])*K_error[2]/(signed int) 10;

    // Clculo do incremento de posio
    delta = (double) Jt[servo][0]*copx_error + (double) Jt[servo][1]*copy_error + (double) Jt[servo][2]*copz_error;
    // Ajuste de Delta
    delta = servo_signal[servo]*delta*K*1e-7;

    // Aplicao do incremento
    applyVelocity(servo,delta);
}


#define MAX_SPEED   50                              // Mxima velocidade (graus/s)
#define MAX_POSVAR  ((double) (MAX_SPEED)*(double) TICK_TIME/1000.0)

// Processamento e aplicao da nova velocidade a aplicar
//
static void applyVelocity(byte servo, double velocity)
{
    // Limitao de velocidade
    if (velocity<-MAX_POSVAR)      velocity = -MAX_POSVAR;
    else if (velocity>MAX_POSVAR)  velocity =  MAX_POSVAR;

    // Actualizao da posio do servo
    servoControl[servo].controller.refInput += velocity;
}


// Controlador Local
//
static void localControl(byte servo)
{
    signed char posReference, y_now;
    signed int error, error_dif, y_vardif;
    double deltaP, deltaI, deltaD, kp, ki, kd;
    far struct_controller *cont=&(servoControl[servo].controller);  // Ponteiro para a estrutura de controlo do servo

    // Limitao da posio referncia
    if ((signed int) cont->refInput<-SERVO_RANGE)      cont->refInput=-(double) SERVO_RANGE;
    else if ((signed int) cont->refInput>SERVO_RANGE)  cont->refInput= (double) SERVO_RANGE;

    posReference=(signed char) cont->refInput;              // Posio desejada (referncia)
    y_now = servo_interface[servo].position;                // Posio efectiva do servo
    
    // Erro actual
    error=(signed int) posReference - (signed int) y_now;

    // Tolerancia de pequenos erros
    if (servoControl[servo].errortol)  {
        // banda morta
        if (error>=-2 && error<=2)  error= 0;
        else if (error== 3)         error= 2;
        else if (error==-3)         error=-2;
    }
    
    // Diferena do erro para a aco proporcional e variao da diferena da sada para a aco derivativa
    error_dif= error-cont->error_prev;
    y_vardif =(signed int) y_now-2*(signed int) cont->y_prev+(signed int) cont->y_prev2;
    
    // Actualizao das sadas/erros anteriores
    cont->y_prev2=cont->y_prev;
    cont->y_prev =y_now;
    cont->error_prev=error;
    
    ki = (double) actuators.control[servo].ki/(double) K_FACTOR;
    kp = (double) actuators.control[servo].kp/(double) K_FACTOR;
    kd = 0.0; //(double) actuators.control[servo].kd/(double) K_FACTOR;

    // Se o controlo de posio em malha fechada no for contemplado passar ao servo seguinte!
    if (ki!=0.0 || kp!=0.0 || kd!=0.0)  {

        // CONTROLO EM MALHA FECHADA DA LOCMOO
        
        // Componentes do compensador
        deltaI = (double) error*ki;                     // Compensao integrativa
        deltaP = (double) error_dif*kp;                 // Compensao proporcional
        deltaD = (double) y_vardif*kd;                  // Compensao derivativa
        
        // Clculo da nova posio a aplicar
        cont->angle += deltaI + deltaP + deltaD;
    }
    else
        cont->angle = cont->refInput;

    // Limitao da posio a solicitar
    if ((signed int) cont->angle<-SERVO_RANGE)      cont->angle=-(double) SERVO_RANGE;
    else if ((signed int) cont->angle>SERVO_RANGE)  cont->angle= (double) SERVO_RANGE;

    // Aplicao do movimento
    servoActuation((signed char) cont->angle,servo);
}
