/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: InitPic.c                                               |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "usart.h"
#include "can.h"

void isr(void);                             // Rotina de servio a interrupes de alta prioridade
void LOWisr(void);

// Quando ocorre um interrupt, o programa "salta" para esta linha do programa,
// que consequentemente reencaminha para a rotina responsvel pelo tratamento das interrupes.
#pragma code isrVector = 0x08
void isrVector(void)
{
_asm
	goto isr			                    // Salta para rotina de interrupt prioritario
_endasm
}

#pragma code isrLowVector = 0x18
void isrLowVector(void)
{
_asm
	goto LOWisr			                    // Salta para rotina de interrupt prioritario
_endasm
}
#pragma code

void initPic(void)
{
    unsigned long i;


    /*------------------------------------------------------------------+
    |                    Definicoes dos TRIS do PIC                     |
    +------------------------------------------------------------------*/
    TRISA = 0b00000000;
    TRISB = 0b00000000;
    TRISC = 0b00001111;

    /*------------------------------------------------------------------+
    |                    Configurao do porto RA                       |
    +------------------------------------------------------------------*/

	ADCON1=0b00000110;      // All RA pins are digital

    /*------------------------------------------------------------------+
    |          			 Inicializao das Comunicaes       		    |
    +------------------------------------------------------------------*/

    usartInit();            // Inicializao da USART
    canInit();              // Inicializao do CAN

    // Espera de 100 ms para garantir que todos os SCU esto
    // em plena operao!
//    wait(1000);

    /*------------------------------------------------------------------+
    |          			 Definies dos Interrupts             		    |
    +------------------------------------------------------------------*/

    // Activacao global das interrupcoes
	RCONbits.IPEN=1;		// Interrupt priority enable
	INTCONbits.GIEH=1;		// Enable high priority interrupts		// -------> alterado (estava comentado)
	INTCONbits.GIEL=1;      // Enable low priority interrupts		// -------> alterado (estava comentado)

	//INTCONbits.PEIE=1;		// Enable all peripheral interrupt sources
	//INTCONbits.GIE =1;      // Enable all interrupt sources
}

// Bloqueio do PIC em <val> ciclos de 100us (para fosc=40MHz)
//
void wait(unsigned int val)
{
    unsigned int i, j;

    for (i=0; i<val; ++i)
        for (j=0; j<140; ++j);
}

// Mquina de estados na troca de mensagens CAN
#define N_OPERATIONS    4           // Nmero de operaes numa transao
#define TIMEOUT         3           // Nmero de tentativas mximo na recepo de uma mensagem
typedef enum {
    SEND_MESSAGE1 = 0,
    RECV_MESSAGE1 = 1,
    SEND_MESSAGE2 = 2,
    RECV_MESSAGE2 = 3
} enum_CanExchange_States;

// Tipos de operaes no agendamente de transmisso pela USART
typedef enum {
    ERROR_MASK      = 0b0001,
    SENDSTREAM_MASK = 0b0010,
    SENDLAST_MASK   = 0b0100,
    SENDTEST_MASK   = 0b1000
} enum_txAgenda;

#pragma interrupt isr nosave=section(".tmpdata")
void isr(void)
{
    // Byte recebido pela USART
    byte rx;
    // Indicadores do estado dos buffer da USART
    bool full, empty;
    // Mscaras para identificao da origem do agendamento de transmisso pela USART
    static enum_txAgenda txAgenda=false;

    //_____ COMUNICAO PELA USART ____________________________________________________

    // Interrupo causada pela receo de um byte pela USART
    if (PIR1bits.RCIF)  {
        // Apenas processa o byte recebido, se no houver nenhuma transmisso agendada,
        if (!txAgenda)  {
            // Verificao da ocorrncia de um Framing Error
            // Este erro  provocado por malformaes da palavra recebida
            if (RCSTAbits.FERR)  {
                RCREG;                                  // Error cleared
                txAgenda|=ERROR_MASK;                   // Agendamento da transmisso de uma mensagem de erro
            }
            // Overrun Error
            // Este erro ocorre quando um caracter  recebido com o buffer
            // de recepo (FIFO) cheio!
            else if (RCSTAbits.OERR)  {
                RCREG;                                  // Remoo de um byte do buffer de recepo da USART
                RCSTAbits.CREN = 0;                     // Error cleared
                RCSTAbits.CREN = 1;
       	    	txAgenda|=ERROR_MASK;                   // Agendamento da transmisso de uma mensagem de erro
   	        }
            // Tudo OK
            else  {
                // Leitura do byte recebido
                rx = RCREG;

                // Em modo de recepo de um stream de bytes...
                if (usartStreamMode() || rx==MESSAGE_REQ)  {
                    // Armazenamento do byte recebido no buffer de recepo
                    usartStoreRx(rx,&full);

                    // Caso o buffer esteja completo, agendar transmisso do stream
                    if (full)  txAgenda|=SENDSTREAM_MASK;
                }
                // Em caso de recepo de um pacote de erro (fora da recepo de streams)
                // agendar a transmisso da ltima mensagem transmitida
                else if (rx==MESSAGE_ERROR)  txAgenda|=SENDLAST_MASK;
                // Em caso de recepo de um pacote de calibrao, enviar trama sequencial
                else if (rx==MESSAGE_TEST)  txAgenda|=SENDTEST_MASK;
            }
        // Caso j haja uma transmisso agendada, descartar byte recebido!
        } else  RCREG;
    }

    // Transmisso de um stream de bytes
    // Aqui no  considerado o semforo de acesso ao buffer de transmisso, dado que a RSI
    // tem prioridade no seu acesso. A funo que quiser modificar este buffer deve tambm
    // verificar se existe alguma transmisso em curso (TXIE)
    if (PIE1bits.TXIE && PIR1bits.TXIF)  {

        usartGetTx((byte *) &TXREG,&empty);

        // Verificao de trmino de transmisso (fim de tx do buffer ou cancelamento da transmisso)
        if (empty)  {
            usartResetTxBuff();                     // Reset do buffer de transmisso
            PIE1bits.TXIE=0;                        // Desactivao do envio por interrupo
        }
    }

    // Atendimento de transmisses agendadas. Apenas o faz quando...
    // ... existe uma transmisso agendada, e no existe nenhuma transmisso em curso.
    if (txAgenda && !PIE1bits.TXIE)  {

        // Transmisso de um stream de bytes
        if (txAgenda & SENDSTREAM_MASK)     usartProcessMsg();

        // Envio de uma mesagem para calibrao das comunicaes
        else if (txAgenda & SENDTEST_MASK)  usartSendTestMsg();

        // Transmisso de uma mensagem de Erro
        else if (txAgenda & ERROR_MASK)     usartSendStatus(MESSAGE_ERROR);

        // No caso de reenvio da ltima mensagem, 
        // suficiente reactivar a transmisso (feito abaixo)

        // Preparao para a recepo de streams
        usartResetRxBuff();                        // Reset do buffer de recepo

        // Clear de agendamento de transmisso
        txAgenda=false;

        // Activao da transmisso
        PIE1bits.TXIE=1;
    }
}

#pragma interruptlow LOWisr //nosave=section(".tmpdata")
void LOWisr(void)
{
    // Variveis para a comunicao CAN
    bool res=true;                                          // Resultado da transmisso
    static byte scu2send=HOST_MCU+1;                        // SCU ID a enviar a mensagem CAN
    static enum_CanExchange_States can_op=SEND_MESSAGE1;    // Mquina de estados

    //_____ COMUNICAO CAN ____________________________________________________________
    // Envio de uma mensagem, via CAN, para um SCU para actualizao dos seus actuadores
    if (PIR1bits.TMR2IF)  {

        PIN_TEST3=1;

        switch (can_op)  {
            // Envio da primeira mensagem de actualizao dos actuadores
            case SEND_MESSAGE1: res=canSendMsg(scu2send,0);
                                break;

            // Recepo de duas mensagens de actualizao dos dados sensoriais
            case RECV_MESSAGE1: res=canReceiveMsg(0);
                                TXB0CONbits.TXREQ=0;
                                break;

            // Envio da segunda mensagem de actualizao dos actuadores
            case SEND_MESSAGE2: res=canSendMsg(scu2send,1);
                                break;

            case RECV_MESSAGE2: res=canReceiveMsg(1);
                                TXB1CONbits.TXREQ=0;
                                break;
        }
        PIN_TEST2=res;

        if ((can_op=(can_op+1)%N_OPERATIONS)==0)  {
            scu2send=((scu2send-HOST_MCU)%N_SCU)+HOST_MCU+1;
            PIN_TEST=!PIN_TEST;
        }

        PIN_TEST3=0;
        PIR1bits.TMR2IF=0;
    }
    //____________________________________________________________________________________
}
