/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: MasterPrinc.c                                           |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "usart.h"

void usartMaster(void);

void main(void)
{	
    // Inicializao das estruturas globais
    initGlobal();

    //inicializacao das configuracoes do MCU
    initPic();
	
    // Comunicaes USART a tempo inteiro
    usartMaster();
}

void usartMaster(void)
{
    // Byte recebido pela USART
    byte rx;
    // Indicadores do estado dos buffer da USART
    bool full;

    // Loop infinito
    for (;;)  {

        // Esperando por um caracter
        while (!PIR1bits.RCIF);
    
        // Verificao da ocorrncia de um Framing Error (provocado por malformaes da palavra recebida)
        // e de Overrun Error (caracter recebido com o buffer de recepo (FIFO) cheio)
        if (RCSTAbits.FERR || RCSTAbits.OERR)  {
            RCREG;                                      // Remoo de um byte do buffer de recepo da USART
            RCSTAbits.CREN = 0;                         // Error cleared
            RCSTAbits.CREN = 1;
            usartSendStatus(MESSAGE_ERROR);             // Preparar mensagem de erro
        }
        // Tudo OK
        else  {
            // Leitura do byte recebido
            rx = RCREG;
    
            // Em modo de recepo de um stream de bytes...
            if (usartStreamMode() || rx==MESSAGE_REQ)  {

                // Se o byte recebido corresponde ao primeiro da stream, ou sendo os seguintes no ocorreu timeout
                if (!usartStreamMode() || !usartCountTimeout())  {

                    // Armazenamento do byte recebido no buffer de recepo
                    usartStoreRx(rx,&full);
        
                    // Caso o buffer esteja completo, agendar transmisso do stream
                    if (full)  {
                        usartProcessMsg();
                    }
                    // caso contrrio, reinicializar a contagem de tempo
                    else  {
                        usartCountTimeStart();
                        continue;
                    }
                }
                else  {
                    PIN_TEST3=1;
                    usartResetRxBuff();
                    continue;
                }
            }
            // Em caso de recepo de um pacote de calibrao, enviar trama sequencial
            else if (rx==MESSAGE_TEST)   usartSendTestMsg();
            // Em caso de recepo de um pacote de erro (fora da recepo de streams)
            // agendar a transmisso da ltima mensagem transmitida
            else if (rx!=MESSAGE_ERROR)  continue;
        }
    
        // Reset do buffer de recepo
        usartResetRxBuff();
    
        // Transmitir consecutivamente todos os bytes do buffer de transmisso
        // A flag TXIF  sempre inicializada a 0 sempre que uma escrita  efectuada no registo TXREG
        while (usartGetTx((byte *) &TXREG,NULL))  {
    
            // Inicializao da contagem de tempo
            usartCountTimeStart();
    
            // Esperar que o byte seja transmitido com sucesso
            while (!PIR1bits.TXIF && !usartCountTimeout());
            // Se o timeout foi atingido, forar sada do ciclo
            if (usartCountTimeout())  {
                PIN_TEST3=1;
                break;
            }
        }
    
        // Reset dos buffer de transmisso e recepo
        usartResetTxBuff();
    }
}
