/*=======================================================================+
| DEPARTAMENTO MECANICA + DEPARTAMENTO DE ELECTRNICA E TELECOMUNICAES |
|                       DA UNIVERSIDADE AVEIRO                           |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              Milton Ruas da Silva                N:21824 (EET)         |
|              David Manuel Costa Gameiro          N:20281 (DEM)         |
|              Filipe Carvalho Mostardinha         N:22085 (DEM)         |
|                                                                        |
| Nome Ficheiro: pic2.h                                                  |
|                                                                        |
|                            HEADER FILE                                 |
+=======================================================================*/

#ifndef _PIC2_H
#define _PIC2_H

#include "global.h"

// Frequncia de funcionamento do CPU (MHz) - 1/4 Fosc
#define F_CPU	    10
// Durao do Tick em milisegundos
#define TICK_TIME   20

// Pinos de interface do PIC
#define SERVO1_PIN	PORTBbits.RB1 //alterado - RB4 //RB1   // Saida de PWM para o servomotor (perodo de 20ms)
#define SERVO2_PIN	PORTBbits.RB4 //alterado - RB5 //RB4   // Saida de PWM para o servomotor
#define SERVO3_PIN	PORTBbits.RB5 //alterado - RB6 //RB5   // Saida de PWM para o servomotor

// Pinos utilizados para efeitos de Debugging
#define TESTPIN 	PORTCbits.RC4           // Execuo da funo main
#define TESTPIN2    PORTCbits.RC5           // Execuo de cdigo aps o fall-down do PWM
#define TESTPIN3    PORTBbits.RB0           // Interrupes de alta frequncia

// _____ MODO DE FUNCIONAMENTO DO PIC _______________________________________________________
#define VPOT_MEASURE    true                // Medir Potencimetro do servo?
#define VPOT_3MOTORS    true                // Medir os trs servos?

// Limiar de deciso para deteco do impulso de corrente acima
// do valor mnimo:
#define VPOT_IMPULSE_LIMIAR	100		// 100/1023*5.0 = 0.488V < 1.0V
#define DUTY_MIN	 980  // 935  // CUIDADO COM ESTE VALOR: O perodo do Timer2 no deve fazer overflow!
#define DUTY_MAX	2020  // 2005

// Propriedades de funcionamento do servo
#define SERVO_RANGE	90              // Gama de funcionamento do servo para cada lado (graus)
#define SERVO_RES   180             // Quantidade de posies possveis: para 90 -> Resoluo=2 (2*RANGE/RES)

// Propriedades dos sensores de Fora
#define FORCE_SENSOR_MIN    40      // Valor mnimo dos sensores de Fora

// Estrutura com os dados I/O de baixo nvel do servo
typedef struct {
    unsigned char actuation;                      // Duty-cycle do PWM
    unsigned char current;            // Largura do impulso de corrente
    signed char   position;         // Voltagem de posio do servo
    signed char   velocity;
} struct_servo;
extern volatile struct_servo servo_interface[N_SERVOS];

// Tick indicador da passagem dos 20ms
extern bool timeTick;

// Os seguintes valores so utilizados para converter uma tenso entre 0 e 5000 num ngulo em graus.
// POS(graus)=ORIGIN-<tenso>*SLOPE/QUOCIENT
#define ORIGIN_0    {247,247,247}
extern volatile signed int origin[N_SERVOS];
// Offset a aplicar  sada dos sensores de fora
#define SPECIAL_0  {0,0,0,0}
extern volatile unsigned char special0[N_SPECIAL_SENSORS];

// ***************
// Device Drivers
// ***************

// Inicializacoes relativas ao microcontrolador
//
extern void initPic(void);

// Funcao para gerar um atraso (ms)
//
void wait(unsigned int value);

// Espera pela passagem de um tick
//
extern void waitTick(void);

// Movimentao do servo para a posio solicitada
//
extern signed char servoActuation(signed char position, byte srv);

// Activao/Desactivao dos PWM's
//
extern void statusPWM(bool status);

// Activao/Desactivao dos Filtros
//
extern void statusFilter(bool status);

// Limitao da posio solicitada
//
extern signed char limitPosition(signed int position);

#endif
