//*****************************************************************************
// COM.C
//
// Dep. Electronica da Universidade de Aveiro
// Jose Luis Azevedo & Milton Ruas da Silva
//
// Mar/2004
//*****************************************************************************
#include <p18f258.h>
#include "com.h"

// Frequncia de funcionamento do CPU (MHz) - 1/4 Fosc
#define F_CPU	10

#define BAUD	115200

#define VAL_NBITS	16		// Numero de bits usado pelo tipo int!

// Funcao que converte um valor de 7 bits num caracter ASCII segundo a ordem '0'~'9','A'...
static unsigned char bin2asc(unsigned char num);

// Funcao para converter 1 caracter ASCII no correspondente digito (at 7 bits)
static unsigned char asc2bin(unsigned char ch);


//*****************************************************************************
// Inicializacao da USART
//
void initUsart()
{
	float spbrg;
// Programacao dos pinos RC7 e RC6
	
	TRISC=(TRISC | 0x80) & 0xBF;
	
// Programacao da Usart (Assincrona por Polling)
 
	spbrg=((float) F_CPU*1e6)/((float) 4*BAUD)-1.0;
	if (spbrg-(unsigned char) spbrg>=0.5)  spbrg+=1.0;

	SPBRG=(unsigned char) spbrg;    // BaudRate
	TXSTAbits.SYNC = 0;             // Modo assincrono
    TXSTAbits.BRGH = 1;	            // BR high speed
	TXSTAbits.TXEN = 1;	            // Transmit Enable 
    RCSTAbits.CREN = 1;	            // Continuous Reception Enable
	RCSTAbits.SPEN = 1;	            // Serial Port Enable
}

//*****************************************************************************
// Funcao para envio de um caracter para a porta serie
//
void sendChar(unsigned char ch)
{
	while( PIR1bits.TXIF == 0 );
	TXREG = ch;
}

//*****************************************************************************
// Funcao para envio de um array para a porta serie
// 
void sendArray(unsigned char *array, unsigned int n)
{
    unsigned int i;

    for (i=0; i<n; ++i)  sendChar(array[i]);
}

//*****************************************************************************
// Funcao para envio da string apontada por "str" para a porta serie
// 
void sendStr(unsigned char *str)
{
	// Todos os caracteres so enviados, incluindo o '\0'
	while(*str!='\0')  sendChar(*(str++));
}

//*****************************************************************************
// Funcao para envio de um valor, sem sinal, para a porta serie, em qq base
// Base representa a base em que o valor deve ser visualizado (base>=2)
// 
void sendVal(unsigned int val, unsigned char base)
{
	unsigned char str[VAL_NBITS];	// Preve a impressao em base 2 para o tipo int (16 bits)
	unsigned char i=0, digito;

	if (base<2)  base=2;

	// Passagem do valor para uma string
	// (cada caracter representa um algarismo na base pretendida)
	do {
		digito = val % base;
		val /= base;
		str[i++] = bin2asc(digito);
	} while( val > 0 );

	// Transmissao dos caracteres
	while (i>0)
		sendChar(str[--i]);	
}

//*****************************************************************************
// Funcao para envio de um valor, com sinal, para a porta serie, em qq base
// 
void sendValSign(signed int val, unsigned char base)
{
	// Se o valor for negativo, transmitir o sinal menos
	if (val<0)  {
		val = -val;
		sendChar('-');
	}

	// Transmitir o mdulo do valor
	sendVal(val,base);
}

//*****************************************************************************
// Funcao para leitura de um caracter da porta serie
//
unsigned char getChar(void)
{
	while( PIR1bits.RCIF == 0 );
	return RCREG;
}

//*****************************************************************************
// Funcao no bloqueante para leitura de um caracter da porta serie
//
unsigned char getKey(void)
{
	if (PIR1bits.RCIF==0)  return 0;
	return RCREG;
}

//*****************************************************************************
// Funcao para leitura de uma cadeia de caracteres (maximo nchar) da porta serie
// 
void getStr(unsigned char *str, unsigned int nchar)
{
	unsigned int count=0;
	unsigned char ch;
	unsigned char backspace[]="\b \b";

	// Leitura de uma sequncia de caracteres at os caracteres
	// '\0','\r' ou '\n' forem digitados.
	while ((ch=getChar())!='\0' && ch!='\r' && ch!='\n')

		switch (ch)  {
		// Limpar todos os caracteres
		case '\t':
			for(; count>0; --count)  sendStr(backspace);
			break;
		// Um caracter para trs
		case '\b':
			if (count>0)  {
				count--;
				sendStr(backspace);
			}
			break;
		default:
			// Introduo de um caracter
			if (count<nchar)  {
				str[count++]=ch;
				sendChar(ch);
			}
			// Buffer de entrada cheio => bell
			else  sendChar('\a');
		}

	// Adio do caracter '\0'
	str[count]=0;
}

//*****************************************************************************
// Funcao para leitura de um valor inteiro com ou sem sinal da porta serie
//
signed int getVal(unsigned char base)
{
	// str possui dois digitos em excesso para incluir o caracter '\0' e o possvel sinal menos.
    unsigned char str[VAL_NBITS+2], *ptr=str;
    unsigned char signal=0, dig;
    signed int val=0;

    getStr(str,VAL_NBITS+1);    // Leitura da string a partir da porta srie

    if (*ptr=='-')  {           // Se o valor e negativo, converter em
        signal=1;               // primeiro o correspondente positivo
        ptr++;
    }

	// Conversao para um valor numerico
	for(; *ptr!='\0'; ptr++)  {
		if ((dig=asc2bin(*ptr))>=base || dig==0xFF)  break;		// Validao do caracter digitado
		val = val*base + (signed int) dig;						// Conversao para valor
	}

	  // Se o valor e negativo, acrescentar sinal
    if (signal)  val = -val;

    return val;
}

//*****************************************************************************
// Funcao para converter 1 digito (at 7 bits) no correspondente caracter ASCII
// na ordem '0'~'9','A','B'...
static unsigned char bin2asc(unsigned char num)
{
	num = (num & 0x7F) + '0';	// MSB ignorado
	if(num > '9')
		num += -('9'+1)+'A';
	return num;
}

//*****************************************************************************
// Funcao para converter 1 caracter ASCII no correspondente digito (at 7 bits)
//
static unsigned char asc2bin(unsigned char ch)
{
    if (ch>='a' && ch<='z')  ch-='a'-'A';

    if (ch>='0' && ch<='9')
        ch -= '0';
    else if (ch>='A' && ch<='Z')
        ch -= 'A'-10;
    else
        ch=0xFF;			// Erro => caracter invalido!

    return ch;
}
