#include "data.h"
#include "mymaths.h"

// Offsets a aplicar  sada dos sensores de fora
#define FORCEOUT_OFFSETX    0
#define FORCEOUT_OFFSETY    0

// Localizao dos sensores de fora
static const signed int sensor_locx[N_SPECIAL_SENSORS] = {-325, 320, -350,  340};     // Coordenada x de cada sensor (*10 mm)
static const signed int sensor_locy[N_SPECIAL_SENSORS] = { 620, 620, -625, -625};     // Coordenada y de cada sensor (*10 mm)

// Massas dos elos em decagramas
#define M1 45
#define M2 45

// Dimenses dos elos mm
#define L1 95
#define L2 95

// Dimenses ao centro de massa de cada elo em mm
#define R1 47
#define R2 47

// Relao de transmisso das correias (*10)
#define NREL    {26, 26, 26}
static const unsigned char Nrel[N_SERVOS]=NREL;

// Offset das posies dos servos
#define SERVO_OFFSET   {0, 0, 70}
static const signed char servo_offset[N_SERVOS]=SERVO_OFFSET;

// Diferena de sinal entre as posies dos servos e do ngulo de trabalho
#define SERVO_SIGNAL  {-1, -1, -1}
static const signed char servo_signal[N_SERVOS]=SERVO_SIGNAL;

// Macros que convertem a posio das juntas em ngulos absolutos, e vice-versa
#define SERVO2THETA_DEG(pos,srv)   ((((signed int) pos-servo_offset[srv])*servo_signal[srv]*10)/(signed int) Nrel[srv])
#define CORRECT_SPEED(vel,srv)     (vel*(double) Nrel[srv]/10.0)


// Clculo do Centro de Presso em dcimas de mm
//
void calcCoP(signed int *copx, signed int *copy)
{
    unsigned char i;
    signed long tempx=0, tempy=0, sumfx=0, sumfy=0;

    // Inicializao dos centros de presso
    *copx=0;
    *copy=0;

    // Clculo do centro de presso em dcimas de milmetro (na componente yy o valor  amplificado)
    for (i=0; i<N_SPECIAL_SENSORS; ++i)  {
        tempx += ((signed long) sensors.special[i]+FORCEOUT_OFFSETX)*(signed long) sensor_locx[i];
        tempy += ((signed long) sensors.special[i]+FORCEOUT_OFFSETY)*(signed long) sensor_locy[i];
        sumfx += (signed long)  sensors.special[i]+FORCEOUT_OFFSETX;
        sumfy += (signed long)  sensors.special[i]+FORCEOUT_OFFSETY;
    }

    // Clculo do centro de presso, se for possvel
    if (sumfx!=0)  *copx=(signed int) (tempx/sumfx);
    if (sumfy!=0)  *copy=(signed int) (tempy/sumfy);
}


void jacobianMatrix(signed int Jt[N_SERVOS][N_SERVOS])
{
    byte i;
    signed int theta[N_SERVOS];
    signed char sin_phi, cos_phi, sin1, cos1, sin12, cos12;
    signed int s2, c2, s12, c12;

    // Adaptao dos ngulos para aplicao na matriz Jacobiana
    for (i=0; i<N_SERVOS; ++i)
        theta[i]=SERVO2THETA_DEG(sensors.servo[i].position,i);
    theta[1]=90-theta[1];

    // Clculo das funes trigonomtricas
    sin_phi=seno(theta[0]);
    cos_phi=coseno(theta[0]);
    sin1=seno(theta[1]);
    cos1=coseno(theta[1]);
    sin12=seno(theta[1]+theta[2]);
    cos12=coseno(theta[1]+theta[2]);

    // Expresses auxiliares
    s2 =(signed int) ((signed long) M2*R2*sin12/(signed long) (M1+M2));
    c2 =(signed int) ((signed long) M2*R2*cos12/(signed long) (M1+M2));
    s12=(signed int) ((((signed long) M1*R1+(signed long) M2*L1)*sin1+(signed long) M2*R2*sin12)/(signed long) (M1+M2));
    c12=(signed int) ((((signed long) M1*R1+(signed long) M2*L1)*cos1+(signed long) M2*R2*cos12)/(signed long) (M1+M2));

    // Matriz Jacobiana transposta
    Jt[0][0]= TRIG_NORMAL( s12,cos_phi);
    Jt[0][1]= 0;
    Jt[0][2]= TRIG_NORMAL(-s12,sin_phi);
    Jt[1][0]= TRIG_NORMAL( c12,sin_phi);
    Jt[1][1]=-s12;
    Jt[1][2]= TRIG_NORMAL( c12,cos_phi);
    Jt[2][0]= TRIG_NORMAL(  c2,sin_phi);
    Jt[2][1]=-s2;
    Jt[2][2]= TRIG_NORMAL(  c2,cos_phi);
/*
    sendValSign(sin_phi,10);
    sendChar(' ');
    sendValSign(cos_phi,10);
    sendChar(' ');
    sendChar(' ');
    sendValSign(s2,10);
    sendChar(' ');
    sendValSign(c2,10);
    sendChar(' ');
    sendValSign(s12,10);
    sendChar(' ');
    sendValSign(c12,10);
    sendChar(' ');
    sendChar(' ');
    sendValSign(Jt[1][0],10);
    sendChar(' ');
    sendValSign(Jt[1][1],10);
    sendChar(' ');
    sendValSign(Jt[1][2],10);
    sendChar(' ');
    sendValSign(Jt[2][0],10);
    sendChar(' ');
    sendValSign(Jt[2][1],10);
    sendChar(' ');
    sendValSign(Jt[2][2],10);
    sendChar('\r');
    sendChar('\n');*/
}
