/*=======================================================================+
| DEPARTAMENTO MECANICA + DEPARTAMENTO DE ELECTRNICA E TELECOMUNICAES |
|                       DA UNIVERSIDADE AVEIRO                           |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              Milton Ruas da Silva                N:21824 (EET)         |
|              David Manuel Costa Gameiro          N:20281 (DEM)         |
|              Filipe Carvalho Mostardinha         N:22085 (DEM)         |
|                                                                        |
| Nome Ficheiro: can.c                                                   |
|                                                                        |
|                                 C FILE                                 |
+=======================================================================*/

#include "can.h"
#include "CanDrivers.h"

// Comprimento das mensagens CAN (bytes)
#define CAN_MESSAGE_LENGTH   8

// Alguns bytes das mensagens CAN (de recepo)
#define CAN_TYPE    0
#define CAN_DATA1   1

// _____ IDENTIFICADOR DAS MENSAGENS CAN ______________________________________________________
// Constantes de referncia ao identificador das mensagens CAN
enum MESSAGE_IDENTIFIER  {
    XCUSRC_MASK    = 0b11110000000,
    XCUDST_MASK    = 0b00001111000,
    CANINDEX_MASK  = 0b00000000100,
    OPERATION_MASK = 0b00000000011,
    CAN_MASK       = XCUDST_MASK | CANINDEX_MASK,

    XCUSRC_LOC     = 7,
    XCUDST_LOC     = 3,
    CANINDEX_LOC   = 2,
    OPERATION_LOC  = 0,

    OP_UPDATE_SENSORS   = 0b00,
    OP_UPDATE_ACTUATORS = 0b01,
    OP_SLAVECAN_STATUS  = 0b10
};

// Extrao de valores do identificador
#define xcuSRC(x)    (((unsigned int) (x) & XCUSRC_MASK   ) >> XCUSRC_LOC   )
#define xcuDST(x)    (((unsigned int) (x) & XCUDST_MASK   ) >> XCUDST_LOC   )
#define canIndex(x)  (((unsigned int) (x) & CANINDEX_MASK ) >> CANINDEX_LOC )
#define operation(x) (((unsigned int) (x) & OPERATION_MASK) >> OPERATION_LOC)

// Construo de padres do identificador
#define xcuSRC_patt(y)    (((unsigned int) (y) << XCUSRC_LOC   ) & XCUSRC_MASK   )
#define xcuDST_patt(y)    (((unsigned int) (y) << XCUDST_LOC   ) & XCUDST_MASK   )
#define canIndex_patt(y)  (((unsigned int) (y) << CANINDEX_LOC ) & CANINDEX_MASK )
#define operation_patt(y) (((unsigned int) (y) << OPERATION_LOC) & OPERATION_MASK)

// Construo do identificador
#define filter(ind)               (xcuDST_patt(HOST_SCU) | canIndex_patt(ind))
#define identifierMSG(op,ind)     (xcuSRC_patt(HOST_SCU) | xcuDST_patt(HOST_MCU) | canIndex_patt(ind) | operation_patt(op))
// ___________________________________________________________________________________________

// PACOTE DE TIPO DAS MENSAGENS OP_UPDATE_SENSORS ____________________________________________
// Constantes de referncia ao byte de tipo dos pacotes de recepo
enum MESSAGE_UPDATE_SENSORS  {
    // Mscaras para extrao de um campo
    SCUSRC_MASK   = 0b11110000,
    MSGINDEX_MASK = 0b00001000,

    // Localizao dos campos no byte de tipo
    SCUSRC_LOC    = 4,
    MSGINDEX_LOC  = 3,

    // Identificao do pacote
    MSGINDEX_1  = 0,
    MSGINDEX_2  = 1
};

// Macros para leitura dos diversos campos no byte de tipo
#define scuSRC_patt(y)   (byte) (((unsigned int) (y) << SCUSRC_LOC  ) & SCUSRC_MASK  )
#define msgIndex_patt(y) (byte) (((unsigned int) (y) << MSGINDEX_LOC) & MSGINDEX_MASK)

// Macros para construo do byte de tipo
#define typePacket(pack_id) (scuSRC_patt(HOST_SCU) | msgIndex_patt(pack_id))
// ___________________________________________________________________________________________________________

// _____ PACOTE DE ESTADO DO SISTEMA _________________________________________________________________
// Pacote de estado do sistema (ltimo byte das primeira mensagem OP_UPDATE_SENSORS)
enum MESSAGE_SYSSTATUS_PACKET {
    // Mscaras dos bits
    PWM_SMASK          = 0b10000000,     // Activao/desactivao dos PWMs
    CALIB_SMASK        = 0b01000000,     // Activao/desactivao da calibrao dinmica
    DEADLINEERROR_MASK = 0b00100000,     // Controlador PID ligado
    MOTIONFINALL_MASK  = 0b00010000,     // Todos os motores terminaram o movimento
    MOTIONFINONE_MASK  = 0b00001000,     // Pelo menos um motor terminou o movimento
    MOTIONFIN2_MASK    = 0b00000100,     // Movimento terminado
    MOTIONFIN1_MASK    = 0b00000010,     // Movimento terminado
    MOTIONFIN0_MASK    = 0b00000001,     // Movimento terminado

    // Localizao dos bits
    PWM_SLOC           = 7,
    CALIB_SLOC         = 6,
    DEADLINEERROR_LOC  = 5,
    MOTIONFINALL_LOC   = 4,
    MOTIONFINONE_LOC   = 3,
    MOTIONFIN2_LOC     = 2,
    MOTIONFIN1_LOC     = 1,
    MOTIONFIN0_LOC     = 0
};

#define motionFinOne (sensors.sysStatus.motionFin[2] | 

// Construo do pacote de estado do sistema a enviar pela USART
#define pwm_patt             (((byte) sensors.sysStatus.pwm << PWM_SLOC) & PWM_SMASK)
#define calib_patt           (((byte) sensors.sysStatus.calib << CALIB_SLOC) & CALIB_SMASK)
#define deadlineError_patt   (((byte) sensors.sysStatus.deadlineError << DEADLINEERROR_LOC) & DEADLINEERROR_MASK)
#define motionFinAll_patt    (((byte) check_motionFinAll() << MOTIONFINALL_LOC) & MOTIONFINALL_MASK)
#define motionFinOne_patt    (((byte) check_motionFinOne() << MOTIONFINONE_LOC) & MOTIONFINONE_MASK)
#define motionFin2_patt      (((byte) sensors.sysStatus.motionFin[2] << MOTIONFIN2_LOC) & MOTIONFIN2_MASK)
#define motionFin1_patt      (((byte) sensors.sysStatus.motionFin[1] << MOTIONFIN1_LOC) & MOTIONFIN1_MASK)
#define motionFin0_patt      (((byte) sensors.sysStatus.motionFin[0] << MOTIONFIN0_LOC) & MOTIONFIN0_MASK)

#define sysStatus_patt      (pwm_patt | calib_patt | deadlineError_patt | motionFinAll_patt | motionFinOne_patt | motionFin2_patt | motionFin1_patt | motionFin0_patt)

// Verifica se pelo menos um motor terminou o movimento
bool check_motionFinOne(void)
{
    byte srv;
    bool res=false;

    for (srv=0; srv<N_SERVOS; ++srv)  res|=sensors.sysStatus.motionFin[srv];
    return res;
}
// Verifica se todos os motores terminaram o movimento
bool check_motionFinAll(void)
{
    byte srv;
    bool res=true;

    for (srv=0; srv<N_SERVOS; ++srv)  res&=sensors.sysStatus.motionFin[srv];
    return res;
}
// ___________________________________________________________________________________________________

// _____ PACOTE DE CONTROLO DE UMA JUNTA _____________________________________________________________
// (Penltimo byte da primeira mensagem OP_UPDATE_ACTUATORS)
enum CONTROL_PACKET  {
    // Mscaras
    PWM_CMASK     = 0b10000000,
    CALIB_CMASK   = 0b01000000,
    CONTROL2_MASK = 0b00110000,
    CONTROL1_MASK = 0b00001100,
    CONTROL0_MASK = 0b00000011,

    // Localizao dos campos
    PWM_CLOC      = 7,
    CALIB_CLOC    = 6,
    CONTROL2_LOC  = 4,
    CONTROL1_LOC  = 2,
    CONTROL0_LOC  = 0
};
#define controlpwm(msg)    (byte) (((msg) & PWM_CMASK) >> PWM_CLOC)
#define controlcalib(msg)  (byte) (((msg) & CALIB_CMASK) >> CALIB_CLOC)
#define controlservo2(msg) (byte) (((msg) & CONTROL2_MASK) >> CONTROL2_LOC)
#define controlservo1(msg) (byte) (((msg) & CONTROL1_MASK) >> CONTROL1_LOC)
#define controlservo0(msg) (byte) (((msg) & CONTROL0_MASK) >> CONTROL0_LOC)

void update_applyControlOn(byte packet)
{
    actuators.sysStatus.pwm=controlpwm(packet);
    actuators.sysStatus.calib=controlcalib(packet);

    actuators.control[2].type=controlservo2(packet);
    actuators.control[1].type=controlservo1(packet);
    actuators.control[0].type=controlservo0(packet);
}
// ___________________________________________________________________________________________________

// _____ PACOTE DE TIPO DAS MENSAGENS OP_SLAVECAN_STATUS _____________________________________________
// Estados de erro relativos ao CAN
enum CAN_ERRORS  {
    CAN_ERROR           = 0b00000001,
    RX_INVALIDMSG_ERROR = 0b00000010,       // Mensagem invlida
    RX_OVERFLOW_ERROR   = 0b00000100,       // Erro de overflow dos buffers de recepo
    RX_INVALID_LENGTH   = 0b00001000,       // Comprimento invlido
    RX_INVALID_ID       = 0b00010000,       // Identificador invlido
    RX_INVALID_DATA     = 0b00100000,       // Dados da mensagem invlidos
    TX_NOTPOSSIBLE      = 0b01000000,       // Transmisso no possvel devido  ocupao de todos os buffers
    TX_ERROR            = 0b10000000,

    TX_BITS = 0b11000000,                   // Bits relativos  transmisso
    RX_BITS = 0b00111111                    // Bits relativos  recepo
};
// ___________________________________________________________________________________________________

// Estrutura descritiva dos erros ocorridos
struct  {
    enum CAN_ERRORS flags;          // Causas
    byte tx_count, rx_count;        // Nmero de ocorrncias
} canErrors={false,0,0};


static void updateServoSensors(byte *message);
static void updateSpecialSensors(byte *message);
static void updateActuators(byte *can_message);     // Actualizao dos dados sensoriais do sistema
static void updateControllers(byte *can_message);   // Actualizao dos parmetros dos controladores
static void setError(enum CAN_ERRORS error);        // Registo de um erro ocorrido com o CAN

// ****************************************************************************

// Inicializao do CAN
//
void initCan(void)
{
    // Programacao dos pinos RB3 e RB2 como entrada e sada respectivamente
	TRISB=(TRISB | 0x08) & 0xFB;

    // Inicializao dos registos do CAN
    myCANInitialize(CAN_MASK,filter(0),filter(1));
}

// Verificao do correcto funcionamento do CAN
//
void checkCan(void)
{
    // O mdulo CAN est bloqueado?
    if (!COMSTATbits.TXBO)  return;

    // Message sending abort
    TXB0CONbits.TXREQ=0;
    TXB1CONbits.TXREQ=0;
    TXB2CONbits.TXREQ=0;

    // Reinicializao do mdulo CAN
    initCan();
}

// Transmisso de uma mensagem CAN relativa aos actuadores do sistema
//
bool sendCanMessage(byte index)
{
    bool res;                           // Resultado da transmisso
    word identifier;                    // Indentificador da mensagem
    byte message[CAN_MESSAGE_LENGTH];   // Mensagem CAN

    // Identificador da mensagem
    identifier=identifierMSG(OP_UPDATE_SENSORS,index);

    // Construo da mensagem a enviar (depende do ndice)
    if (index==0)  {
        updateServoSensors(message);
        res=myCANSendMessage0(identifier,message,CAN_MESSAGE_LENGTH);
    }
    else  {
        updateSpecialSensors(message);
        res=myCANSendMessage1(identifier,message,CAN_MESSAGE_LENGTH);
    }

    // Transmisso da mensagem
    if (!res)  setError(TX_NOTPOSSIBLE);

    // Apenas retorna true, se a transmisso teve sucesso!
    return res;
}


// ****************************************************************************
// Recepo de uma mensagem CAN de um dos SCU
//
bool receiveCanMessage(byte buffer)
{
    bool res;                           // Resultado da recepo
    word id;
    byte length;
    byte message[CAN_MESSAGE_LENGTH];
    enum CAN_STATUS_RX warnings;
    enum CAN_ERRORS errors=false;    

    // Leitura do buffer de recepo:
    // Buffer origem  o 0
    if (buffer==0)  {
        if ((res=myCANReceiveMessage0(&id, message, &length, &warnings)))  {
            updateActuators(message);               // Actualizao dos actuadores
        }
    }
    // Buffer origem  o 1
    else  {
        if ((res=myCANReceiveMessage1(&id, message, &length, &warnings)))  {
            updateControllers(message);             // Actualizao dos parmetros dos controladores
        }
    }

    // Erro de Overflow
    if (warnings & CAN_STATUS_RXOVERFLOW)    errors|=RX_OVERFLOW_ERROR;
    // Erro de mensagem invlida
    if (warnings & CAN_STATUS_RXINVALIDMSG)  errors|=RX_INVALIDMSG_ERROR;
    // Se ocorreram erros assinal-los!
    if (errors) setError(errors);

    return res;
}


// ****************************************************************************
// Registo de um erro ocorrido com o CAN
//
static void setError(enum CAN_ERRORS error)
{
    // Erros ocorridos
    canErrors.flags|=error;
    
    // Actualizao dos contadores de erros
    if (canErrors.tx_count<0xFF && (error & TX_BITS))  canErrors.tx_count++;
    if (canErrors.rx_count<0xFF && (error & RX_BITS))  canErrors.rx_count++;
}


// ****************************************************************************
// Actualizao dos dados sensoriais do sistema
//
static void updateActuators(byte *can_message)
{
    byte i;

    // Actualizar as posies e velocidades dos servos da junta
    for (i=0; i<N_SERVOS; ++i)  {
        // Actualizao da posio e da velocidade a solicitar
        actuators.servo[i].position=can_message[i];
        actuators.servo[i].velocity=can_message[i+N_SERVOS];
    }
    // Estado on/off dos controladores
    update_applyControlOn(can_message[2*N_SERVOS]);

    // Actualizao do Kd do terceiro motor
    actuators.control[N_SERVOS-1].kd=can_message[2*N_SERVOS+1];
/*
    // REMOVER
    for (i=0; i<N_SERVOS; ++i)  {
        sensors.servo[i].position=actuators.servo[i].position;
        sensors.servo[i].velocity=actuators.servo[i].velocity;
        sensors.sysStatus.controlOn[i] = actuators.control[i].controlOn;
    }

    for (i=0; i<N_SERVOS && actuators.servo[i].position==-128; ++i);
    sensors.sysStatus.reset = (i==N_SERVOS);
    // REMOVER
*/
}

// ****************************************************************************
// Actualizao dos parmetros dos controladores
//
static void updateControllers(byte *can_message)
{
    byte i;

    // Actualizao dos parmetros dos dois primeiros motores
    for (i=0; i<N_SERVOS-1; ++i)  {
        actuators.control[i].ki=can_message[i];
        actuators.control[i].kp=can_message[i+N_SERVOS];
        actuators.control[i].kd=can_message[i+2*N_SERVOS];
    }
    // Actualizao do kp e kd do terceiro motor
    actuators.control[N_SERVOS-1].ki=can_message[  N_SERVOS-1];
    actuators.control[N_SERVOS-1].kp=can_message[2*N_SERVOS-1];
}


static void updateServoSensors(byte *message)
{
    byte i, ind=0;

    // Preenchimento do primeiro pacote
    // Byte de tipo
    message[ind++]=typePacket(MSGINDEX_1,0,0,0);
    for (i=0; i<N_SERVOS; ++i,++ind)  {
        message[ind]=sensors.servo[i].position;
        message[N_SERVOS+ind]=sensors.servo[i].velocity;
    }

    ind+=N_SERVOS;      // Actualizao do ndice da mensagem CAN tendo em conta as velocidades introduzidas

    // Pacote de estado do sistema
    message[ind++]=sysStatus_patt;
}

static void updateSpecialSensors(byte *message)
{
    byte i, ind=0;

    // Preenchimento do segundo pacote
    // Byte de tipo
    message[ind++]=typePacket(MSGINDEX_2);
    // Dados das correntes consumidas pelos servos
    for (i=0; i<N_SERVOS; ++i,++ind)
        message[ind]=sensors.servo[i].current;

    // Sensores especiais
    for (i=0; i<N_SPECIAL_SENSORS; ++i,++ind)
        message[ind]=sensors.special[i];

    // Finalizao do preenchimento com zeros
    while (ind<CAN_MESSAGE_LENGTH)  message[ind++]=0;
}
