/*=======================================================================+
| DEPARTAMENTO MECANICA + DEPARTAMENTO DE ELECTRNICA E TELECOMUNICAES |
|                       DA UNIVERSIDADE AVEIRO                           |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              Milton Ruas da Silva                N:21824 (EET)         |
|              David Manuel Costa Gameiro          N:20281 (DEM)         |
|              Filipe Carvalho Mostardinha         N:22085 (DEM)         |
|                                                                        |
| Nome Ficheiro: pic2.c                                                  |
|                                                                        |
|                               C FILE                                   |
+=======================================================================*/

#include "pic2.h"
#include "can.h"
#include "mymaths.h"
#include <string.h>

// Endereo de cada motor no multiplexer
#define MUX_INPUT(servo)    ((N_SERVOS-1-(servo)) & 0b11)
// Canais do multiplexer aos sensores especiais
static const unsigned char MUX_INPUT_EXTRA[N_SPECIAL_SENSORS]={6, 7, 14, 15};
#define SELECT_MUX_CHANNEL(ch)  PORTAbits.RA1=(ch) & 1;      \
                                PORTAbits.RA2=((ch)>>1) & 1; \
                                PORTAbits.RA3=((ch)>>2) & 1; \
                                PORTAbits.RA5=((ch)>>3) & 1;

// Offset a aplicar  sada dos sensores de fora
volatile struct_sensoradapt special0[N_SPECIAL_SENSORS]=SPECIAL0_DATA;

// Os seguintes valores so utilizados para converter a sada da ADC num ngulo em graus.
volatile struct_sensoradapt origin[N_SERVOS]=ORIGIN_0;

// Macro conversora de ngulo para unidades de PWM
#define PWM(ang)    (byte) ((SERVO_RANGE-(signed int) (ang))*SERVO_RES/(2*(signed int) SERVO_RANGE))

// ______ VARIVEIS ESTTICAS E GLOBAIS _____________________________________________________

// Estrutura com os valores finais das medies (para cada ciclo de PWM)
volatile struct_servo2 servo_interface[N_SERVOS];

// Estrutura com os dados temporrios das medies/actuaes
static struct  {
    unsigned char which;                        // Seleco do servo
    struct {
        bool pwmOn;                             // PWM ligado?
        unsigned char pwm;                      // Largura do PWM
	    unsigned char vpotImp;                  // Durao do impulso de corrente
	    unsigned int  vpotMin, vpotMin_prev;    // Tenso mnima actual e anterior (relacionada com a posio)
    } data[N_SERVOS];
} servo;

// Estruturas utilizadas para o filtro
typedef struct {
    signed int val_max;
    signed int val_min;
    signed int val_prev;
    signed int valnew_prev;
} struct_filterVars;
typedef struct {
    struct_filterVars vpot;
    struct_filterVars pos;            // Posio
    struct_filterVars vel;            // Velociodade
    struct_filterVars curr;           // Corrente
} struct_filterServo;
static struct_filterServo sfilter[N_SERVOS];

// Estrutura com dados necessrios  medio de velocidade e corrente
#define N_POS   5                               // Nmero de medies a armazenar no array circular

static struct {
    struct_servo data[N_SERVOS][N_POS];         // Dados do Array circular para todos os servos
    signed char index;                          // ndice do array de posies onde est armazenado a medio mais antiga
} ring;

// Tick temporal indicativo da passagem de um ciclo de PWM (20 ms)
bool timeTick=false;


// _____ CONSTANTES PARA CONFIGURAO DOS TIMERS ____________________________________________

// O Timer 0  usado para fazer aquisio do sinal proveniente do potencimetro
#define T0CON_VAL   0b11000011      // Configurao do Timer0 (Prescaler=16)
#define T0_PRESC	16			    // Prescaler (alterar o programa)
#define T0_OUT		250  	        // Perodo do Timer (us)
static const unsigned char kt0=(unsigned char) (255-(unsigned int) F_CPU*T0_OUT/T0_PRESC+11);
static const unsigned char countmax=((unsigned int) TICK_TIME*1000-((unsigned int) DUTY_MAX-DUTY_MIN))/((unsigned int) T0_OUT*N_SERVOS);

// O Timer 1  utilizado para definir o perodo do PWM para o servo
#define T1CON_VAL   0b00100001      // Configurao do Timer1 (Prescaler=4)
#define T1_PRES     4				// Prescaler seleccionado na programacao do timer 1 (alterar funo initPit(...))
#define T1_OUT	    TICK_TIME       // Perodo do Timer 1 (ms)
#define KT1		    (65535-((unsigned int) F_CPU*1000/T1_PRES)*T1_OUT+1) // Constante de diviso p/ o Timer 1
static const unsigned char kt1H=(unsigned char) (KT1 >> 8);
static const unsigned char kt1L=(unsigned char) (KT1 & 0xFF);
#define TIMER1_CNT  ((word) TMR1L | (word) TMR1H<<8)       // Valor actual do timer1

    // Valores do contador do Timer1 para definio de margens de segurana
static const unsigned int  kt1sec  = 65535-(unsigned int) F_CPU*T0_OUT/T1_PRES+1;           // Margem de segurana antes do rise up do PWM
static const unsigned int  kt1max  = KT1+(unsigned int) F_CPU*(DUTY_MIN-T0_OUT)/T1_PRES+1;  // Margem de segurana antes do PWM falldown zone
static const unsigned int  kt1str  = KT1+(unsigned int) F_CPU*DUTY_MIN/T1_PRES+1;           // Valor do timer1 imediatamente antes do PWM falldown zone
static const unsigned int  kt1min  = KT1+(unsigned int) F_CPU*DUTY_MAX/T1_PRES+1;           // Margem de segurana depois do PWM falldown zone

typedef enum {
    LOW=0,
    READY=1,
    HIGH=2
} enum_pwmState;
static enum_pwmState pwmState=LOW;

// O Timer 2 define o duty-cycle do PWM
#define T2CON_STD   0b01001101;     // Postacaler 1:10, Prescaler 1:4, TMR2ON=true
#define T2CON_HF    0b00000100;     // Configurao para as interrupes de alta frequncia
#define T2_PRES		4               // Prescaler do Timer 2 (alterar funo initPit(..))
#define T2_POST     10              // Postscaler do Timer 2 (alterar funo initPit(..))
#define T2_RES_FACTOR 10            // Factor de preciso do passo de variao do PWM
#define T2_OUT_HF   (unsigned char) (((unsigned int) DUTY_MAX-DUTY_MIN)*T2_RES_FACTOR/(unsigned int) SERVO_RES)
static const unsigned char kt2_1st = (unsigned char) ((unsigned int) DUTY_MIN*F_CPU/(T2_PRES*T2_POST)-4);
static const unsigned char kt2_hf  = (unsigned char) ((unsigned int) T2_OUT_HF*F_CPU/T2_RES_FACTOR-1);

// ______ DEFINIO DAS FUNES DE SERVIO A INTERRUPES ___________________________________
void highISR(void);						// Prottipo da funo de servio a interrupes
void lowISR(void);

// Interrupt de alta prioridade
#pragma code InterruptVectorHigh = 0x08	//Quando ocorre um interrupt o programa "salta" para este endereo
void InterruptVectorHigh (void)
{
    _asm  goto highISR  _endasm
}

// Interrupt de baixa prioridade
#pragma code InterruptVectorLow = 0x18	//Quando ocorre um interrupt o programa "salta" para este endereo
void InterruptVectorLow (void)
{
    _asm  goto lowISR  _endasm
}
#pragma code

// Inicializao das variveis globais do mdulo PIC2 e SERVO
//
static void initLocal(void)
{
    unsigned char srv, i;

    memset((void *) servo_interface,0,sizeof(struct_servo)*N_SERVOS);
    memset((void *) sfilter,0,sizeof(struct_filterServo)*N_SERVOS);
    memset((void *) &ring,0,sizeof(ring));

    // Seleco de servo
    servo.which=0;
    for (srv=0; srv<N_SERVOS; ++srv)  {

        // Dados temporrios das medies
        servo.data[srv].pwmOn=false;
        servo.data[srv].pwm=PWM(0);
        servo.data[srv].vpotMin=0xFFFF;
        servo.data[srv].vpotMin_prev=0;
        servo.data[srv].vpotImp=0;
    }
}


// ==========================================================================================
// Device Drivers
// ==========================================================================================

// Inicializacoes relativas ao microcontrolador
//
void initPic(void)
{
    byte i;

    // Inicializao das variveis globais
    initLocal();

    // Programao dos portos de entrada/sada
	TRISA=0b00000001;		// RA0 input / RA1..RA7 outputs
	TRISB=0b00000000;		// RB0...RB7 outputs
	TRISC=0b00001111;		// RC0...RC4 outputs; RC3..RC0 inputs

    // Inicializao dos pinos de sada
    PORTA=MUX_INPUT(servo.which)<<1;  // Seleco da entrada do multiplexer

    SERVO1_PIN=SERVO2_PIN=SERVO3_PIN=0;     // Pinos de PWM
    TESTPIN=TESTPIN2=TESTPIN3=0;   // Pinos de teste

	// Programao do Timer 2 (desligado e interrupes desactivadas)

    TMR2  = 0;
   	PR2   = kt2_1st;        // Perodo standard
    T2CON = T2CON_STD;      // Postacaler 1:10, Prescaler 1:4, TMR2ON=true
	IPR1bits.TMR2IP=1;		// High priority interrupt
   	PIE1bits.TMR2IE=0;      // Interrupes do Timer2 desactivadas

    // Programacao do Timer 1

    TMR1H = kt1H;  		    // Period
	TMR1L = kt1L;

    T1CON = T1CON_VAL;

	IPR1bits.TMR1IP=1;		// Low priority interrupt
	PIE1bits.TMR1IE=1;		// Timer1 interrupt enable

    // Programacao do Timer 0

    TMR0L = kt0;              // Contador inicializado

    T0CON = T0CON_VAL;

	INTCON2bits.TMR0IP=1;	// High priority interrupt
    INTCONbits.TMR0IE=VPOT_MEASURE;     // Interrupes activadas

    // Programacao da ADC

	ADCON0=0b10000001;      // Fosc/64, channel 0(RA0), enable ADC converter
	ADCON1=0b01001110;      // Left justified; RA0 analog; the rest are digital

	IPR1bits.ADIP=1;		// Low Priority interrupt
	PIE1bits.ADIE=0;		// Low Priority interrupt

	// Inicializao das comunicaes CAN
	initCan();
    // Interrupes (recepo) de baixa prioridade
    IPR3bits.RXB0IP=IPR3bits.RXB1IP=0;
    // Activa os interrupts dos buffers recepcao CAN
    PIE3bits.RXB0IE=PIE3bits.RXB1IE=1;  // Interrupts Enable

    initUsart();

    // Activacao global das interrupcoes
	RCONbits.IPEN=1;		// Interrupt priority enable
	INTCONbits.GIEH=1;		// Enable high priority interrupts
	INTCONbits.GIEL=1;      // Enable low priority interrupts
}


// Funcao para gerar um atraso (ms)
//
void wait(unsigned int value)
{
	unsigned int i, end=value/TICK_TIME;

	for (i=0; i<end; ++i)  waitTick();
}

// Espera pela passagem de um tick
//
void waitTick(void)
{
    while (timeTick==0);
    timeTick=0;
}

// Movimentao do servo para a posio solicitada
//
signed char servoActuation(signed char position, byte srv)
{
    // Correco da posio solicitada
    position=limitPosition(position);

    // Posio a solicitar
    servo_interface[srv%N_SERVOS].actuation=position;
    // PWM associado  posio
    servo.data[srv%N_SERVOS].pwm=PWM(position);

    return position;
}

// Activao/Desactivao dos PWM's
//
bool statusPWM(bool status, byte srv)
{
    byte i;
    bool res;

    // Inicializao das flags de movimento geral
    sensors.sysStatus.pwm=false;
    sensors.sysStatus.motionFinAll=true;
    sensors.sysStatus.motionFinOne=false;

    // Inicializao do primeiro servo a utilizar
    for (res=true,i=0; i<N_SERVOS; ++i)  {

        // Actualizao da flag de PWM activo
        // Se o PWM for desactivado, activar a flag de finalizao do movimento
        if (srv==-1 || i==srv)  {
            res&=servo.data[i].pwmOn=(status & 1) & abs(actuators.servo[i].position)<=SERVO_RANGE;
            if (!servo.data[i].pwmOn)  sensors.sysStatus.motionFin[i]=true;
        }

        // Actualizao das flags do byte SysStatus:
        sensors.sysStatus.pwm|=servo.data[i].pwmOn;
        sensors.sysStatus.motionFinAll&=sensors.sysStatus.motionFin[i];     // Todos os motores terminaram o movimento
        sensors.sysStatus.motionFinOne|=sensors.sysStatus.motionFin[i];     // Um dos motores terminaram o movimento
    }

    return res;
}

// Limitao da posio solicitada
//
signed char limitPosition(signed int position)
{
    // Correco da posio solicitada
    if (position<-SERVO_RANGE)       position=-SERVO_RANGE;
    else if (position>SERVO_RANGE)   position= SERVO_RANGE;

    return (signed char) position;
}

// ==========================================================================================
// Funes relativas a interrupes
// ==========================================================================================


// Actualizao das medidas
void sampleExtraSensors(void);
void updateServoMeasures(void);
void finalizeServoMeasures(void);

// Rotina de servio s interrupes de alta prioridade
//
// O pragma seguinte indica que a rotina definida atende interrupes de alta prioridade,
// com uma seco de dados prpria com o nome isr_tmp, e que no  necessrio guardar o
// contexto da seco de dados geral.
#pragma interrupt highISR nosave=section(".tmpdata")
void highISR(void)
{
	unsigned char counter;
    unsigned int tmr1_val;

    // Interrupo responsvel por arrancar a ADC
    if (INTCONbits.TMR0IE && INTCONbits.TMR0IF)  {
        // Arranque do timer0 para provocar um delay antes da prxima medio
        TMR0L=kt0;

        // Apenas prossegue se existe tempo para executar as operaes associadas a este interrupt
        tmr1_val =TIMER1_CNT;
        if ((!PIE1bits.TMR1IE || !PIR1bits.TMR1IF) && (!PIE1bits.TMR2IE || !PIR1bits.TMR2IF)
            && (tmr1_val<=kt1sec) && (tmr1_val<=kt1max || tmr1_val>=kt1min))  {

            TESTPIN2=1;

            // Arranque da ADC para leitura do sensor do servo
            PIR1bits.ADIF=0;
            PIE1bits.ADIE=1;
            ADCON0bits.GO=1;
        }
        INTCONbits.TMR0IF=0;
    }

    // Interrupo responsvel por ler os sensores e efectuar uma transao pelo CAN
    if (PIE1bits.ADIE && PIR1bits.ADIF)  {

        // Actualizao das medidas
        updateServoMeasures();

        TESTPIN2=0;

        PIE1bits.ADIE=0;
        PIR1bits.ADIF=0;
    }

    // Interrupo responsvel por baixar o PWM
    if (PIE1bits.TMR2IE && PIR1bits.TMR2IF)  {

        // O Timer2  redefinido de modo a provocar interrupes de alta frequncia
        T2CON=T2CON_HF;
        PR2=kt2_hf;

        // PWM na zona de descida
        pwmState=HIGH;
        counter=0;
        do  {
            while (PIR1bits.TMR2IF==0);
            PIR1bits.TMR2IF=0;
            
            if (SERVO1_PIN && counter>=servo.data[0].pwm)  SERVO1_PIN=0;    // Desactivar PWM?
            if (SERVO2_PIN && counter>=servo.data[1].pwm)  SERVO2_PIN=0;
            if (SERVO3_PIN && counter>=servo.data[2].pwm)  SERVO3_PIN=0;
        } while (counter++<SERVO_RES);

        TESTPIN=1;

        // Reset das definies do Timer2
        T2CON = T2CON_STD;      // Postacaler 1:10, Prescaler 1:4, TMR2ON=true
   	    PR2   = kt2_1st;        // Perodo standard
        PIE1bits.TMR2IE=0;      // Interrupes do Timer2 desactivadas

        // Clculo dos valores sensoriais definitivos para um ciclo de PWM
        finalizeServoMeasures();

        // Barramento CAN operacional?
        //checkCan();

        // Amostragem dos sensores especiais
        sampleExtraSensors();

        // Actualizao dos time Ticks temporais (apenas se o PWM estiver a ser gerado)
    	timeTick=true;
        // PWM desactivado
        pwmState=LOW;

        // Reinicio das medies dos servos
        INTCONbits.TMR0IF=1;

        TESTPIN=0;

        PIR1bits.TMR2IF=0;
	}

    // Interrupo responsvel por elevar o PWM
    if (PIE1bits.TMR1IE && PIR1bits.TMR1IF)  {

        TMR1H=kt1H;             // Inicializao do contador do Timer 1
        TMR1L=kt1L;

        // Inicializao do Timer2 para gerar o primeiro interrupt de alta frequncia
        TMR2=0;
       	PIR1bits.TMR2IF=0;
       	PIE1bits.TMR2IE=1;		// Activao das interrupes do Timer2

        // Se a gerao de PWM estiver autorizada, faz-lo:
        if (servo.data[0].pwmOn)  SERVO1_PIN=1;
        if (servo.data[1].pwmOn)  SERVO2_PIN=1;
        if (servo.data[2].pwmOn)  SERVO3_PIN=1;

        // PWM prestes a entrar na zona de descida
        pwmState=READY;

        INTCONbits.TMR0IF=1;    // Executar imediatamente uma medio do potencimetro

		PIR1bits.TMR1IF=0;      // Reset Timer1 interrupt flag
	}
}

// O cdigo definido no interior deste pragma indica que a seco de variveis temporrias
// a utilizar NO  a geral, mas sim a mesma que  utilizada pela RSI de alta prioridade.
#pragma tmpdata highISR_tmp

static void delay(unsigned char value);

void sampleExtraSensors(void)
{
    unsigned char i;
	unsigned int val;
    signed int difference;

    for (i=0; i<N_SPECIAL_SENSORS; ++i)  {
        
        SELECT_MUX_CHANNEL(MUX_INPUT_EXTRA[i]); // Seleco do canal do multiplexer a ler
        delay(1);                       // Delay de 100us para estabilizao do valor de sada

        ADCON0bits.GO=1;                // Inicio de conversao da ADC
        while (ADCON0bits.GO);          // Esperando pelo fim da conversao

        // Leitura do valor convertido
        val = (unsigned int) ADRESH;

		// The following operations must be executed one by one, to allow the successfull compilation!
		// The reason is related to the maximum size of the MATHS section, which must be bigger if complex calculous are done.
        val /= 2;                   				// Estas operaes aritmticas em separado destinam-se
        val += (unsigned char) special0[i].offset;	// a aumentar o espao de dados MATHS
		val *= (unsigned char) special0[i].slope;
		val /= 100;
        
		sensors.special[i]=(unsigned char) val;
    }

    // Preparao do multiplexer para leitura dos potencimetros de posio
    servo.which=0;
    SELECT_MUX_CHANNEL(MUX_INPUT(servo.which));
    delay(1);
}

// Clculo dos valores sensoriais definitivos para um ciclo de PWM
//
static signed int filter(struct_filterVars *f, signed int y);

void updateServoMeasures(void)
{
    // Leitura da tenso presente no potencimetro do servo
    unsigned int vpot=((unsigned int) ADRESH<<2) | (ADRESL>>6);

	if (vpot<servo.data[servo.which].vpotMin)
        servo.data[servo.which].vpotMin=vpot;  // Actualizao do valor mnimo

    // Medio da largura do impulso de corrente
	if (vpot>=(servo.data[servo.which].vpotMin_prev+VPOT_IMPULSE_LIMIAR))
        servo.data[servo.which].vpotImp++;

    // Transferncia da entrada do multiplexer para o servo seguinte
    if (VPOT_3MOTORS)  {
        if (++servo.which>=N_SERVOS)  servo.which=0;
        SELECT_MUX_CHANNEL(MUX_INPUT(servo.which));
    }
}

void finalizeServoMeasures(void)
{
    byte srv, i;

    byte current;
    word current_sum;
    signed int vel_sum;
	signed char pos, pos_old, vel;
    signed long tmp;
    struct_servo *ptr;

    //sendChar('\r');
    //sendChar('\n');

  	// Passagem dos valores finais
    for (srv=0; srv<N_SERVOS; ++srv)  {

        //sendVal((unsigned int) servo.data[srv].vpotMin,10);
        //sendChar(' ');

        // Sada mnima do potencimetro (em graus)
        tmp= -(signed long) servo.data[srv].vpotMin;
        tmp*=(signed long) origin[srv].slope;
        tmp/=(signed long) 1000;
        tmp+=(signed long) origin[srv].offset;

        if (tmp<-127)     tmp=-127;         // Truncagem do resultado de modo a caber numa varivel de 8 bits
        else if (tmp>127) tmp= 127;

        pos=(signed char) tmp;              // Valor final

        // Largura do impulso de corrente
        current=(byte) ((word) servo.data[srv].vpotImp*100/(word) countmax);

        // Clculo da velocidade
        ptr=ring.data[srv];
        pos_old=ptr[ring.index].position;
        tmp-=(signed long) pos_old;
        if (tmp<-127)     tmp=-127;         // Truncagem do resultado de modo a caber numa varivel de 8 bits
        else if (tmp>127) tmp= 127;
        vel=(signed char) tmp;              // Valor final

        // Actualizao do array circular
        ptr[ring.index].position=pos;
        ptr[ring.index].velocity=vel;
        ptr[ring.index].current =current;
        ring.index=(ring.index+1)%N_POS;

        // Mdia deslizante da velocidade
        for (i=0, vel_sum=0; i<N_POS; ++i)  vel_sum+=(signed int) ptr[i].velocity;
        vel=(signed char) (vel_sum/(signed int) N_POS);

        // Mdia deslizante da corrente
        for (i=0, current_sum=0; i<N_POS; ++i)  current_sum+=(word) ptr[i].current;
        current=(byte) (current_sum/(word) N_POS);

        // Filtragem dos dados sensoriais
        pos=(signed char) filter(&sfilter[srv].pos,(signed int) pos);       // Posio
        vel=(signed char) filter(&sfilter[srv].vel,(signed int) vel);       // Velocidade
        current=(byte) filter(&sfilter[srv].curr,(signed int) current);     // Corrente

        // Actualizao dos dados locais
        servo_interface[srv].position=pos;
        servo_interface[srv].velocity=vel;
        servo_interface[srv].current =current;

        // Actualizao das estruturas globais
        sensors.servo[srv].position=servo_interface[srv].position;          // Posio angular
        sensors.servo[srv].velocity=servo_interface[srv].velocity;          // Velocidade (graus/N_POS ticks)
        sensors.servo[srv].current =servo_interface[srv].current;           // Corrente mdia deslizante

	    // Reinicializao das variveis auxiliares
	    servo.data[srv].vpotMin_prev=servo.data[srv].vpotMin;
	    servo.data[srv].vpotMin=-1;
	    servo.data[srv].vpotImp=0;
    }
}


static signed int filter(struct_filterVars *f, signed int y)
{
    signed int y_new, ynew_prev, y_prev, y_max, y_min;

    //return y;

    // Leitura das variveis de estado
    ynew_prev=f->valnew_prev;       // Sada anterior filtrada
    y_prev=f->val_prev;             // Sada anterior
    y_max =f->val_max;              // Amplitude mxima
    y_min =f->val_min;              // Amplitude mnima

    // Limitao da sada
    if (y>y_max)        y_new=y_max;
    else if (y<y_min)   y_new=y_min;
    else                y_new=y;

    // Actualizao das amplitudes limite
    if (y>y_prev)  {
        y_max=y;
        y_min=y_prev;
    }
    else  {
        y_max=y_prev;
        y_min=y;
    }

    // Actualizao da sada anterior
    y_prev=y;

    // Filtro Passa-baixo
    y_new = (signed char) (((signed int) y_new+ynew_prev)/2);
    ynew_prev = y_new;

    // Reposio das variveis de estado
    f->valnew_prev=ynew_prev;
    f->val_prev=y_prev;
    f->val_max =y_max;
    f->val_min =y_min;

    return y_new;
}

//*******************************************************************************************
// Funcao para gerar um atraso (multiplos de 0,1 ms)
//
static void delay(unsigned char value)
{
	unsigned char i,k;

	for(k=0;k<value;k++)		    // Wait value*100us
		for(i=0;i<7*F_CPU;i++)  Nop();  // Ciclo de espera de aprox. 100 uS
}
#pragma tmpdata


// Rotina de atendimento s interrupes de baixa prioridade
#pragma interruptlow lowISR save=section(".tmpdata")
void lowISR(void)
{
    enum_pwmState state_prev;

    TESTPIN3=1;
    if (PIR3bits.RXB0IF)  {

        state_prev=pwmState;
        TXB1CONbits.TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
        receiveCanMessage(0);
        if (state_prev==LOW && pwmState==LOW)  sendCanMessage(0);
    }

    if (PIR3bits.RXB1IF)  {

        state_prev=pwmState;
        TXB0CONbits.TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
        receiveCanMessage(1);
        if (state_prev==LOW && pwmState==LOW)  sendCanMessage(1);
    }
    TESTPIN3=0;
}
