/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: InitPic.c                                               |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "usart.h"
#include "can.h"

// Quando ocorre um interrupt, o programa "salta" para esta linha do programa,
// que consequentemente reencaminha para a rotina responsvel pelo tratamento das interrupes.
#ifdef MCHP_C18

void isr(void);                             // Rotina de servio a interrupes de alta prioridade
void lowisr(void);

#pragma code isrVector = 0x08
void isrVector(void)
{
_asm
	goto isr			                    // Salta para rotina de interrupt prioritario
_endasm
}
#pragma code lowisrVector = 0x18
void lowisrVector(void)
{
_asm
	goto lowisr			                    // Salta para rotina de interrupt prioritario
_endasm
}
#pragma code

#endif

void initPic(void)
{
    /*------------------------------------------------------------------+
    |                    Definicoes dos TRIS do PIC                     |
    +------------------------------------------------------------------*/
    TRISA = 0b00000000;
    TRISB = 0b00000000;
    TRISC = 0b00001111;

    /*------------------------------------------------------------------+
    |                    Configurao do porto RA                       |
    +------------------------------------------------------------------*/

	ADCON1=0b00000110;      // All RA pins are digital

    /*------------------------------------------------------------------+
    |                    Inicializao de pinos                         |
    +------------------------------------------------------------------*/

    PIN_TEST=PIN_TEST2=PIN_TEST3=0;

    /*------------------------------------------------------------------+
    |          			 Inicializao das Comunicaes       		    |
    +------------------------------------------------------------------*/

    usartInit();            // Inicializao da USART
    canInit();              // Inicializao do CAN

    // Espera de 100 ms para garantir que todos os SCU esto
    // em plena operao!
//    wait(1000);

    /*------------------------------------------------------------------+
    |          			 Definies dos Interrupts             		    |
    +------------------------------------------------------------------*/

    // Activacao global das interrupcoes
	RCON_IPEN=1;		// Interrupt priority enable
	INTCON_GIEH=1;		// Enable high priority interrupts		// -------> alterado (estava comentado)
	INTCON_GIEL=1;      // Enable low priority interrupts		// -------> alterado (estava comentado)

	//INTCON_PEIE=1;		// Enable all peripheral interrupt sources
	//INTCON_GIE =1;      // Enable all interrupt sources
}

// Bloqueio do PIC em <val> ciclos de 100us (para fosc=40MHz)
//
void wait(unsigned int val)
{
    unsigned int i, j;

    for (i=0; i<val; ++i)
        for (j=0; j<140; ++j);
}

// Mquina de estados na troca de mensagens CAN
#define N_OPERATIONS    2           // Nmero de operaes numa transao
typedef enum {
    SEND_MESSAGE1 = 0,
    SEND_MESSAGE2 = 1
} enum_CanExchange_States;


#ifdef MCHP_C18
#pragma interrupt isr save=section(".tmpdata")
void isr(void)
#else
void interrupt isr(void)
#endif
{
    // Indicador se uma mensagem deve ser transmitida
    bool ready=false;
    byte rx;

    // Esperando por um caracter
    if (PIR1_RCIF)  {

        // Verificao da ocorrncia de um Framing Error (provocado por malformaes da palavra recebida)
        // e de Overrun Error (caracter recebido com o buffer de recepo (FIFO) cheio)
        if (RCSTA_FERR || RCSTA_OERR)  {
            RCREG;                                      // Remoo de um byte do buffer de recepo da USART
            RCSTA_CREN = 0;                         // Error cleared
            RCSTA_CREN = 1;
        }
        else  {
            rx=RCREG;

            // Se a receber uma stream de bytes e ocorreu timeout, limpar buffer de recepo
            // Este procedimento deve ser feito antes de avaliar o novo caracter recebido!
            if (usartStreamMode() && usartCountTimeout())
                usartResetRxBuff();
            
            // Em modo de recepo de um stream de bytes...
            if (usartStreamMode() || rx==MESSAGE_REQ || rx==MESSAGE_TEST)  {
            
                // Armazenamento do byte recebido no buffer de recepo
                usartStoreRx(rx,&ready);
            
                // Caso o buffer esteja ainda incompleto, reinicializar a contagem de tempo,
                // e esperar por mais um caracter
                if (!ready)  usartCountTimeStart();
            }
            
            // Se o buffer de recepo ainda no est cheio, voltar ao modo de recepo
            if (ready)  {
                
                // Aps este ponto, a transmisso da mensagem de resposta est em mente!
                
                // Processar mensagem de resposta
                usartProcessMsg();
                
                // Limpar buffer de recepo
                usartResetRxBuff();
                
                // Transmitir consecutivamente todos os bytes do buffer de transmisso
                // A flag TXIF  sempre inicializada a 0 sempre que uma escrita  efectuada no registo TXREG
                while (usartGetTx((byte *) &TXREG,NULL))  {
                
                    // Inicializao da contagem de tempo
                    usartCountTimeStart();
                
                    // Esperar que o byte seja transmitido com sucesso
                    while (!PIR1_TXIF && !usartCountTimeout());
                    // Se o timeout foi atingido, forar sada do ciclo
                    if (usartCountTimeout())  break;
                }
                
                // Reset dos buffer de transmisso e recepo
                usartResetTxBuff();
            }
        }
    }
}

#ifdef MCHP_C18
#pragma interruptlow lowisr save=section(".tmpdata")
void lowisr(void)
#else
void interrupt low_priority lowisr(void)
#endif
{
    // Variveis para a comunicao CAN
    bool res=true;                                          // Resultado da transmisso
    static byte scu2send=HOST_MCU+1;                        // SCU ID a enviar a mensagem CAN
    static enum_CanExchange_States can_op=SEND_MESSAGE1;    // Mquina de estados
/*
    if (INTCON_TMR0IE && INTCON_TMR0IF)  {

        usartCountTimeStart();               // Inicializao do valor inicial do Timer0
        PIN_TEST3=!PIN_TEST3;
    }
*/
    //_____ COMUNICAO CAN ____________________________________________________________
    // Envio de uma mensagem, via CAN, para um SCU para actualizao dos seus actuadores
    if (PIR1_TMR2IF)  {

        switch (can_op)  {
            // Envio da primeira mensagem de actualizao dos actuadores
            case SEND_MESSAGE1: TXB1CON_TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
                                res=canReceiveMsg(1);
                                canSendMsg(scu2send,0);
                                break;

            // Envio da segunda mensagem de actualizao dos actuadores
            case SEND_MESSAGE2: TXB0CON_TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
                                res=canReceiveMsg(0);
                                canSendMsg(scu2send,1);
                                break;
        }
        PIN_TEST2=res;

        if ((can_op=(can_op+1)%N_OPERATIONS)==0)  {
            scu2send=((scu2send-HOST_MCU)%N_SCU)+HOST_MCU+1;
            PIN_TEST=!PIN_TEST;
        }

        PIR1_TMR2IF=0;
    }
}
