#include "data.h"
#include "mymaths.h"
#include "pic2.h"

// Offsets a aplicar  sada dos sensores de fora
#define FORCEOUT_OFFSETX    0
#define FORCEOUT_OFFSETY    (-100)

// Localizao dos sensores de fora
static const signed int sensor_locx[N_SPECIAL_SENSORS] = {-33, 33, -33,  33};     // Coordenada x de cada sensor (mm)
static const signed int sensor_locy[N_SPECIAL_SENSORS] = { 62, 62, -62, -62};     // Coordenada y de cada sensor (mm)

// Massas dos elos em decagramas
#define M1 45
#define M2 45

// Dimenses dos elos em mm
#define L0 20               // Distancia da junta transversal para a longitudinal
#define L1 95               // Elo inferior da perna
#define L2 95               // Elo superior da perna

// Dimenses ao centro de massa de cada elo em mm
#define R1 47
#define R2 47

// Macros que convertem a posio das juntas em ngulos absolutos, e vice-versa
#define SERVO2THETA_DEG(pos,srv)   ((((signed int) pos-servo_offset[srv])*servo_signal[srv]*10)/(signed int) Nrel[srv])


// Clculo do Centro de Presso em dcimas de mm
//
void calcCoP(signed int cop[3])
{
    unsigned char i;
    signed long tempx=0, tempy=0, sumfx=0, sumfy=0;
    signed int theta[N_SERVOS];

    // Clculo do centro de presso em dcimas de milmetro (na componente yy o valor  amplificado)
    for (i=0; i<N_SPECIAL_SENSORS; ++i)  {
        tempx += ((signed long) sensors.special[i]+FORCEOUT_OFFSETX)*(signed long) sensor_locx[i];
        tempy += ((signed long) sensors.special[i]+FORCEOUT_OFFSETY)*(signed long) sensor_locy[i];
        sumfx += (signed long)  sensors.special[i]+FORCEOUT_OFFSETX;
        sumfy += (signed long)  sensors.special[i]+FORCEOUT_OFFSETY;
    }

    // Inicializao dos centros de presso
    cop[0]=0;
    cop[1]=0;
    // Clculo do centro de presso, se for possvel
    if (sumfx!=0)  cop[0]=(signed int) (tempx/sumfx);
    if (sumfy!=0)  cop[1]=(signed int) (tempy/sumfy);

    // Clculo dos ngulos dos elos (no os dos servos)
    for (i=0; i<N_SERVOS; ++i)
        theta[i]=SERVO2THETA_DEG(servo_interface[i].position,i);

    // Clculo da altura em milimetros
    cop[2] =(signed int) L1+L2;
    cop[2]-=(signed int) (((signed long) L1*coseno(theta[1])+(signed long) L2*coseno(theta[1]+theta[2]))
                          *coseno(theta[0])/(signed long) 1000000);
/*
    sendValSign(cop[0],10);
    sendChar(' ');
    sendValSign(cop[1],10);
    sendChar(' ');
    sendValSign(cop[2],10);
    sendChar('\r');
    sendChar('\n');*/
}

// Clculo da Inclinao em graus
//
void calcInc(signed int inc[3])
{
    double aux;
	
	aux = (double) (( (double) (sensors.special[2]-128)*168.78*3.14)/180);		// j est multiplicado por 100 para ser compativel com a tabela do asin
	inc[0] = (signed int) (aseno((signed int) aux));
	
	aux = (double) (( (double) (sensors.special[0]-128)*168.78*3.14)/180);		// j est multiplicado por 100 para ser compativel com a tabela do asin
	inc[1] = (signed int) (aseno((signed int) aux));
	
	inc[2] = 0;
}

// Mximo ngulo theta1+theta2 para evitar a singularidade no controlo de altura
#define SINGULARITY_PROTECTION  10

void jacobianMatrix(signed int Jt[N_SERVOS][N_SERVOS])
{
    byte i,j;
    signed int theta[N_SERVOS];
    signed int sin_phi, cos_phi, sin1, cos1, sin12, cos12;
    signed long s2, c2, s12, c12;

    // Adaptao dos ngulos para aplicao na matriz Jacobiana
    for (i=0; i<N_SERVOS; ++i)
        theta[i]=SERVO2THETA_DEG(servo_interface[i].position,i);

    // Clculo das funes trigonomtricas
    sin_phi=seno(theta[0]);
    cos_phi=coseno(theta[0]);
    sin1=seno(theta[1]);
    cos1=coseno(theta[1]);
    sin12=seno(theta[1]+theta[2]);
    cos12=coseno(theta[1]+theta[2]);

    // Expresses auxiliares
    s2 =(signed long) M2*R2*sin12/(signed long) (M1+M2);
    c2 =(signed long) M2*R2*cos12/(signed long) (M1+M2);
    s12=(((signed long) M1*R1+(signed long) M2*L1)*sin1+(signed long) M2*R2*sin12)/(signed long) (M1+M2);
    c12=(((signed long) M1*R1+(signed long) M2*L1)*cos1+(signed long) M2*R2*cos12)/(signed long) (M1+M2);

    // Matriz Jacobiana transposta
    Jt[0][0]= (signed int) ( c12*cos_phi/(signed long) 10000);
    Jt[1][0]= (signed int) (-s12*sin_phi/(signed long) 10000);
    Jt[2][0]= (signed int) ( -s2*sin_phi/(signed long) 10000);
    Jt[0][1]= 0;
    Jt[1][1]= (signed int) (-c12/(signed long) 10);
    Jt[2][1]= (signed int) ( -c2/(signed long) 10);

    s2 = (signed long) L2*sin12;
    s12= (signed long) L1*sin1+(signed long) L2*sin12;
    c12= (signed long) L1*cos1+(signed long) L2*cos12;

    Jt[0][2]= (signed int) (-c12*sin_phi/(signed long) 10000);
    Jt[1][2]= (signed int) (-s12*cos_phi/(signed long) 10000);
    Jt[2][2]= (signed int) ( -s2*cos_phi/(signed long) 10000);
}
