/*-----------------------------------------------------------------------+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|                                                                        |
| Nome:                                                                  |
|       David Manuel Costa Gameiro          N:20281                      |
|       Filipe Carvalho Mostardinha         N:22085                      |
|                                                                        |
| Data:   10/Maio/2004     Nome Ficheiro: InitCan.c                      |
+-----------------------------------------------------------------------*/

#include "can.h"
#include "usart.h"
#include "CanDrivers.h"

// Comprimento das mensagens CAN
#define CAN_MESSAGE_LENGTH   8

// Alguns bytes das mensagens CAN (de recepo)
#define CAN_TYPE    0
#define CAN_DATA1   1

// _____ IDENTIFICADOR DAS MENSAGENS CAN ______________________________________________________
// Constantes de referncia ao identificador das mensagens CAN
enum MESSAGE_IDENTIFIER  {
    XCUSRC_MASK    = 0b11110000000,
    XCUDST_MASK    = 0b00001111000,
    CANINDEX_MASK  = 0b00000000100,
    OPERATION_MASK = 0b00000000011,
    CAN_MASK       = XCUDST_MASK | CANINDEX_MASK,

    XCUSRC_LOC     = 7,
    XCUDST_LOC     = 3,
    CANINDEX_LOC   = 2,
    OPERATION_LOC  = 0,

    OP_UPDATE_SENSORS   = 0b00,
    OP_UPDATE_ACTUATORS = 0b01,
    OP_SLAVECAN_STATUS  = 0b10
};

// Extrao de valores do identificador
#define xcuSRC(x)    (((unsigned int) (x) & XCUSRC_MASK   ) >> XCUSRC_LOC   )
#define xcuDST(x)    (((unsigned int) (x) & XCUDST_MASK   ) >> XCUDST_LOC   )
#define canIndex(x)  (((unsigned int) (x) & CANINDEX_MASK ) >> CANINDEX_LOC )
#define operation(x) (((unsigned int) (x) & OPERATION_MASK) >> OPERATION_LOC)

// Construo de padres do identificador
#define xcuSRC_patt(y)    (((unsigned int) (y) << XCUSRC_LOC   ) & XCUSRC_MASK   )
#define xcuDST_patt(y)    (((unsigned int) (y) << XCUDST_LOC   ) & XCUDST_MASK   )
#define canIndex_patt(y)  (((unsigned int) (y) << CANINDEX_LOC ) & CANINDEX_MASK )
#define operation_patt(y) (((unsigned int) (y) << OPERATION_LOC) & OPERATION_MASK)

// Construo do identificador
#define filter(ind)               (xcuDST_patt(HOST_MCU) | canIndex_patt(ind))
#define identifierMSG(op,ind,dst) (xcuSRC_patt(HOST_MCU) | xcuDST_patt(dst) | canIndex_patt(ind) | operation_patt(op))
// ___________________________________________________________________________________________

// _____ PACOTE DE TIPO DAS MENSAGENS OP_UPDATE_SENSORS _______________________________________________
// Constantes de referncia ao byte de tipo das mensagens de actualizao de sensores (operao)
enum MESSAGE_UPDATE_SENSORS  {
    // Mscaras para extrao de um campo
    SCUSRC_MASK   = 0b11110000,         // 0b1111xxxx: id do scu fonte
    MSGINDEX_MASK = 0b00001000,         // 0bxxxx1xxx: ndice da mensagem de actualizao dos sensores

    // Localizao dos campos no byte de tipo
    SCUSRC_LOC    = 4,
    MSGINDEX_LOC  = 3,

    // Identificao do pacote
    MSGINDEX_1 = 0,                     // Primeira mensagem
    MSGINDEX_2 = 1                      // Segunda mensagem
};

// Macros para leitura dos diversos campos no byte de tipo
#define scuSRC(x)   (((unsigned int) (x) & SCUSRC_MASK  ) >> SCUSRC_LOC  )
#define msgIndex(x) (((unsigned int) (x) & MSGINDEX_MASK) >> MSGINDEX_LOC)
// ___________________________________________________________________________________________________


// _____ PACOTE DE CONTROLO DE UMA JUNTA _____________________________________________________________
// (Penltimo byte da primeira mensagem OP_UPDATE_ACTUATORS)
enum CONTROL_PACKET  {
    // Mscaras
    PWM_MASK      = 0b10000000,
    CALIB_MASK    = 0b01000000,
    CONTROL2_MASK = 0b00110000,
    CONTROL1_MASK = 0b00001100,
    CONTROL0_MASK = 0b00000011,

    // Localizao dos campos
    PWM_LOC       = 7,
    CALIB_LOC     = 6,
    CONTROL2_LOC  = 4,
    CONTROL1_LOC  = 2,
    CONTROL0_LOC  = 0
};
#define controlpwm_patt(scu)    (((unsigned char) actuators[scu].sysStatus.pwm    << (PWM_LOC     )) & (PWM_MASK     ))
#define controlcalib_patt(scu)  (((unsigned char) actuators[scu].sysStatus.calib  << (CALIB_LOC   )) & (CALIB_MASK   ))
#define controlservo2_patt(scu) (((unsigned char) actuators[scu].control[2].type  << (CONTROL2_LOC)) & (CONTROL2_MASK))
#define controlservo1_patt(scu) (((unsigned char) actuators[scu].control[1].type  << (CONTROL1_LOC)) & (CONTROL1_MASK))
#define controlservo0_patt(scu) (((unsigned char) actuators[scu].control[0].type  << (CONTROL0_LOC)) & (CONTROL0_MASK))

#define control_patt(scu) ((controlpwm_patt(scu)) | (controlcalib_patt(scu)) | (controlservo0_patt(scu)) | (controlservo1_patt(scu)) | (controlservo2_patt(scu)))
// ___________________________________________________________________________________________________

// _____ PACOTE DE TIPO DAS MENSAGENS OP_SLAVECAN_STATUS _____________________________________________
// Estados de erro relativos ao CAN
enum CAN_ERRORS  {
    CAN_ERROR           = 0b00000001,
    RX_INVALIDMSG_ERROR = 0b00000010,       // Mensagem invlida
    RX_OVERFLOW_ERROR   = 0b00000100,       // Erro de overflow dos buffers de recepo
    RX_INVALID_LENGTH   = 0b00001000,       // Comprimento invlido
    RX_INVALID_ID       = 0b00010000,       // Identificador invlido
    RX_INVALID_DATA     = 0b00100000,       // Dados da mensagem invlidos
    TX_NOTPOSSIBLE      = 0b01000000,       // Transmisso no possvel devido  ocupao de todos os buffers
    TX_ERROR            = 0b10000000,

    TX_BITS = 0b11000000,                   // Bits relativos  transmisso
    RX_BITS = 0b00111111                    // Bits relativos  recepo
};
// ___________________________________________________________________________________________________

// Estrutura descritiva dos erros ocorridos
volatile struct  {
    enum CAN_ERRORS flags;          // Causas
    byte tx_count, rx_count;        // Nmero de ocorrncias
} canErrors={false,0,0};

static bool updateActuators(byte scu_id, byte *can_message);        // Construo de uma mensagem para actualizao dos actuadores
static bool updateControllers(byte scu_id, byte *can_message);      // Construo de uma mensagem para actualizao dos controladores
static bool updateServoSensors(byte scu_id, byte *can_message);     // Actualizao dos dados sensoriais dos servos
static bool updateSpecialSensors(byte scu_id, byte *can_message);   // Actualizao dos dados sensoriais dos sensores extra
static void setMcuError(enum CAN_ERRORS error);                     // Registo de um erro ocorrido com o CAN


// ****************************************************************************
// Configurao do Timer2 para transmisso peridica de mensagens CAN
#define TIMER2_PRES_CODE    0b10
#define TIMER2_POST_CODE    0b0001
#define TIMER2_PRES 16
#define TIMER2_POST 2
#define TIMER2_OUT  500         // Periodo em microsegundos
#define TIMER2_PER  (byte) ((word) TIMER2_OUT*F_CPU/((word) TIMER2_PRES*TIMER2_POST)-1)

// Inicializao do CAN
//
void canInit(void)
{
    // Programacao dos pinos RB3 e RB2 como entrada e sada respectivamente
	TRISB=(TRISB | 0x08) & 0xFB;

    // Inicializao dos registos do CAN
    myCANInitialize(CAN_MASK,filter(0),filter(1));

    // Interrupes (recepo) de baixa prioridade
    IPR3bits.RXB0IP=0;
    IPR3bits.RXB1IP=0;
    // Activa os interrupts dos buffers recepcao CAN
    PIE3bits.RXB0IE=1;                                  // Interrupts Enable
    PIE3bits.RXB1IE=1;
    PIR3bits.RXB0IF=0;
    PIR3bits.RXB1IF=0;

    // Configurao do Timer2 para gerao de interrupes de 1000 em 1000 us
    // Este interrupt ser usado para transmisses de mensagens CAN
    T2CONbits.TOUTPS3=(TIMER2_POST_CODE>>3) & 1;
    T2CONbits.TOUTPS2=(TIMER2_POST_CODE>>2) & 1;
    T2CONbits.TOUTPS1=(TIMER2_POST_CODE>>1) & 1;
    T2CONbits.TOUTPS0=TIMER2_POST_CODE & 1;                 // Postscaler

    T2CONbits.T2CKPS1=(TIMER2_PRES_CODE >> 1) & 1;          // Prescaler
    T2CONbits.T2CKPS0=TIMER2_PRES_CODE & 1;

    PR2=TIMER2_PER;             // Definio do perodo
    TMR2=0;                     // Inicializao do contador
    T2CONbits.TMR2ON=1;         // Timer2 ON
    IPR1bits.TMR2IP=0;          // Interrupo de alta prioridade
    PIE1bits.TMR2IE=1;          // Activao das interrupes
    PIR1bits.TMR2IF=0;          // Activao das interrupes

    // Activao das interrupes globais, no mduloPIC.H/C
}

// ****************************************************************************
// Transmisso de uma mensagem CAN relativa  actualizao dos actuadores do sistema
//
bool canSendMsg(byte scu_id, byte index)
{
    bool res;                           // Resultado da transmisso
    unsigned int identifier;            // IDentificador da mensagem
    byte message[CAN_MESSAGE_LENGTH];   // Mensagens CAN

    // Validao do identificador do SCU
    if (scuIndex(scu_id)>=N_SCU)  {
        setMcuError(RX_INVALID_ID);
        return false;
    }

    // Identificador da primeira mensagem
    identifier=identifierMSG(OP_UPDATE_ACTUATORS,index,scu_id);

    // Construo da mensagem a enviar
    if (index==0)  {
        updateActuators(scu_id,message);
        res=myCANSendMessage0(identifier,message,CAN_MESSAGE_LENGTH);
    }
    else  {
        updateControllers(scu_id,message);
        res=myCANSendMessage1(identifier,message,CAN_MESSAGE_LENGTH);
    }

    // Caso o envio falhe, notificar o erro ao PC
    if (!res)  setMcuError(TX_NOTPOSSIBLE);

    // Retorno do sucesso/falha da transmisso
    return (res);
}


// ****************************************************************************
// Recepo de uma mensagem CAN de um dos SCU, relativa  actualizao dos sensores
//
bool canReceiveMsg(byte buffer)
{
    bool res;                                       // Resultado da recepo
    word identifier;                                // Identificador da mensagem recebida
    byte length;                                    // Comprimento, em bytes, da mensagem
    far byte message[CAN_MESSAGE_LENGTH];               // Mensagem CAN recebida
    enum CAN_STATUS_RX warnings;                    // Avisos sobre a recepo
    enum CAN_ERRORS    errors=false;                // Erros no CAN

    // Buffer origem  o 1
    if (buffer==1)  {
        if ((res=myCANReceiveMessage1(&identifier, message, &length, &warnings)))
            updateSpecialSensors(xcuSRC(identifier),message);
    }
    // Buffer origem  o 0
    else  {
        if ((res=myCANReceiveMessage0(&identifier, message, &length, &warnings)))
            updateServoSensors(xcuSRC(identifier),message);
    }

    // Erro de Overflow
    if (warnings & CAN_STATUS_RXOVERFLOW)    errors|=RX_OVERFLOW_ERROR;
    // Erro de mensagem invlida
    if (warnings & CAN_STATUS_RXINVALIDMSG)  errors|=RX_INVALIDMSG_ERROR;
    // Se ocorreram erros, assinal-los!
    if (errors)  setMcuError(errors);

    return res;
}

// Clear do estado de erros
void canClearStatus(void)
{
    canErrors.flags = false;

    canErrors.tx_count = 0;
    canErrors.rx_count = 0;
}

// ****************************************************************************
// Registo de um erro ocorrido com o CAN
//
static void setMcuError(enum CAN_ERRORS error)
{
    byte buffer[4];

    // Erros ocorridos
    canErrors.flags|=error;
    
    // Actualizao dos contadores de erros
    if (canErrors.tx_count<0xFF && (error & TX_BITS))  canErrors.tx_count++;
    if (canErrors.rx_count<0xFF && (error & RX_BITS))  canErrors.rx_count++;

    // Preenchimento da mensagem a enviar pela USART
    buffer[0]=canErrors.flags;
    buffer[1]=canErrors.tx_count;
    buffer[2]=canErrors.rx_count;
    buffer[3]=HOST_MCU;

    // Armazenamento da mensagem num buffer externo
    // S quando for pedido, ela circular para o PC
    usartStore2extBuff(buffer,4);
}

// ****************************************************************************
// Construo da mensagem a enviar ao slave para actualizao dos actuadores
//
static bool updateActuators(byte scu_id, byte *can_message)
{
    byte i;
    byte scu=scuIndex(scu_id);
    far struct_paramControl *ptr=0;

    // Preenchimento do pacote a enviar
    for (i=0; i<N_SERVOS; ++i)  {
        switch (actuators[scu].control[i].type)  {
            // Controlo do centro de presso
            case COP_CONTROL:   ptr=&refControl[scu].cop[i];
                                break;
            // Controlo de inclinao
            case INC_CONTROL:   ptr=&refControl[scu].inc[i];
                                break;
            // Controlo de giroscpios
            case GIRO_CONTROL:  ptr=&refControl[scu].giro[i];
                                break;
            // Controlo dos servos
            default:            ptr=&refControl[scu].servo[i];
        }

        can_message[i]=ptr->position;
        can_message[N_SERVOS+i]=ptr->velocity;
    }

    // Byte de Estado on/off a aplicar nos controladores
    can_message[2*N_SERVOS]=control_patt(scu);

    // Se o nmero de servos for 3, utilizar o ltimo byte para conter
    // o parmetro Ki do terceiro servo
    can_message[2*N_SERVOS+1]=actuators[scu].control[N_SERVOS-1].kd;

    return true;
}


// ****************************************************************************
// Construo da mensagem a enviar ao slave para actualizao dos actuadores
//
static bool updateControllers(byte scu_id, byte *can_message)
{
    byte i;
    byte scu=scuIndex(scu_id);

    // Preenchimento do pacote a enviar
    // Kp's, Kd's e Ki's dos dois primeiros servos...
    for (i=0; i<N_SERVOS-1; ++i)  {
        can_message[i]=actuators[scu].control[i].ki;
        can_message[N_SERVOS+i]=actuators[scu].control[i].kp;
        can_message[2*N_SERVOS+i]=actuators[scu].control[i].kd;
    }
    // Kp's e Kd's do terceiro servo...
    can_message[  N_SERVOS-1]=actuators[scu].control[N_SERVOS-1].ki;
    can_message[2*N_SERVOS-1]=actuators[scu].control[N_SERVOS-1].kp;

    return true;
}


// ****************************************************************************
// Actualizao dos dados sensoriais do sistema
//
static bool updateServoSensors(byte scu_id, byte *can_message)
{
    byte i;
    byte scu=scuIndex(scu_id);

    // Se o pacote recebido corresponde ao primeiro...
    // Actualizar as posies e velocidades dos servos da junta
    for (i=0; i<N_SERVOS; ++i)  {
        sensors[scu].servo[i].position=can_message[CAN_DATA1+i];
        sensors[scu].servo[i].velocity=can_message[CAN_DATA1+N_SERVOS+i];
    }
    // Actualizao do estado do sistema
    sensors[scu].sysStatus=can_message[CAN_DATA1+2*N_SERVOS];
}

static bool updateSpecialSensors(byte scu_id, byte *can_message)
{
    byte i;
    byte msg_type=can_message[CAN_TYPE];
    byte scu=scuIndex(scu_id);

    // Se o pacote recebido corresponde ao segundo (ltimo)...
    // Actualizar as correntes consumidas pelos servos da junta
    for (i=0; i<N_SERVOS; ++i)
        sensors[scu].servo[i].current=can_message[CAN_DATA1+i];

    // Actualizao dos sensores especiais
    for (i=0; i<N_SPECIAL_SENSORS; ++i)
        sensors[scu].special[i]=can_message[CAN_DATA1+N_SERVOS+i];

    return true;
}
