/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: Tipos.h                                                 |
|                                                                        |
|                            HEADER FILE                                 |
+=======================================================================*/


#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <p18f258.h>
#include "types.h"

#define HOST_MCU    0   // ID deste MCU

// Frequncia de funcionamento do CPU (MHz) - 1/4 Fosc
#define F_CPU	10

// Dados sobre o robot
#define N_SCU           8       // Nmero de Slave Controller Units
#define N_SERVOS        3       // Nmero de servos por SCU
#define N_SPECIAL_SENSORS 4     // Nmero de sensores de fora por p

// Posies standard de todos os servos
#define STD_POSITIONS  {{  0,  0,-70},      /* Perna direita  */ \
                        {  0,  0, 70},      /* Perna esquerda */ \
                        {  0,  0,  0},      /* Anca direita   */ \
                        {  0,  0,  0},      /* Anca esquerda  */ \
                        {  0,  0,  0},      /* Tronco         */ \
                        {  0,-70,-70},      /* Brao direito  */ \
                        {  0, 70, 70},      /* Brao esquerdo */ \
                        { 70,  0,  0}}      /* Cabea         */

// Clculo do ndice dos arrays dos sensores/actuadores correspondentes ao ID do SCU
#define scuIndex(scu_id)    ((scu_id)-HOST_MCU-1)

// Tipo enumerado do gnero de controlador implementado em cada junta
typedef enum {
    NO_CONTROL  = 0b00,          // Funcionamento em malha aberta
    COP_CONTROL = 0b01,          // Controlo do Centro de Presso
    INC_CONTROL = 0b10,          // Controlo de inclinao
    GIRO_CONTROL= 0b11           // Controlo de giroscpios
} enum_controlType;

// Estrutura descritiva de um servo (parte sensorial)
typedef struct {
    signed char position;               // Posio
    signed char velocity;               // Velocidade
    unsigned char current;              // Corrente consumida
} struct_servo;

// Estrutura descritiva dos valores referncia a aplicar a um SCU
typedef struct {
    signed char position;               // Posio
    signed char velocity;               // Velocidade
} struct_paramControl;
typedef struct {
    struct_paramControl servo[N_SERVOS];
    struct_paramControl cop[N_SERVOS];
    struct_paramControl inc[N_SERVOS];
    struct_paramControl giro[N_SERVOS];
 } struct_refControl;

// Estrutura descritiva dos sensores
typedef struct {
    byte sysStatus;                             // Estado do sistema
    struct_servo servo[N_SERVOS];               // Sensores dos servos
    unsigned char special[N_SPECIAL_SENSORS];   // Sensores de fora dos ps
} struct_sensors;

typedef struct {
    // Estrutura de actuao de status para cada SCU
    struct {
        bool pwm;                               // PWM on/off
        bool calib;                            // Calib on/off
    } sysStatus;
    // Estrutura de Controlo para cada junta
    struct {
        byte ki, kp, kd;
        enum_controlType type;
    } control[N_SERVOS];
} struct_actuators;

// Variveis globais descritivas do sistema
#pragma varlocate 1 sensors
extern volatile struct_sensors sensors[N_SCU];                          // Sensores

#pragma varlocate 2 actuators
extern volatile struct_actuators actuators[N_SCU];                      // Actuadores

#pragma varlocate 3 refControl
extern volatile struct_refControl refControl[N_SCU];

extern void initGlobal(void);

extern void initSensors(void);

extern void initActuators(void);

#endif
