/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: InitPic.c                                               |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "usart.h"
#include "can.h"

void highISR(void);                             // Rotina de servio a interrupes de alta prioridade
void lowISR(void);

// Quando ocorre um interrupt, o programa "salta" para esta linha do programa,
// que consequentemente reencaminha para a rotina responsvel pelo tratamento das interrupes.
#pragma code highVector = 0x08
void interrupt_highVector(void)
{
_asm
	goto highISR			                    // Salta para rotina de interrupt prioritario
_endasm
}

#pragma code lowVector = 0x18
void interrupt_lowVector(void)
{
_asm
	goto lowISR			                    // Salta para rotina de interrupt prioritario
_endasm
}

#pragma code

void initPic(void)
{
    /*------------------------------------------------------------------+
    |                    Definicoes dos TRIS do PIC                     |
    +------------------------------------------------------------------*/
    TRISA = 0b00000000;
    TRISB = 0b00000000;
    TRISC = 0b00001111;

    /*------------------------------------------------------------------+
    |                    Configurao do porto RA                       |
    +------------------------------------------------------------------*/

	ADCON1=0b00000110;      // All RA pins are digital

    /*------------------------------------------------------------------+
    |                    Inicializao de pinos                         |
    +------------------------------------------------------------------*/

    PIN_TEST=PIN_TEST2=PIN_TEST3=0;

    /*------------------------------------------------------------------+
    |          			 Inicializao das Comunicaes       		    |
    +------------------------------------------------------------------*/

    usartInit();            // Inicializao da USART
    canInit();              // Inicializao do CAN

    // Espera de 100 ms para garantir que todos os SCU esto
    // em plena operao!
//    wait(1000);

    /*------------------------------------------------------------------+
    |          			 Definies dos Interrupts             		    |
    +------------------------------------------------------------------*/

    // Activacao global das interrupcoes
	RCONbits.IPEN=1;		// Interrupt priority enable
	INTCONbits.GIEH=1;		// Enable high priority interrupts		// -------> alterado (estava comentado)
	INTCONbits.GIEL=1;      // Enable low priority interrupts		// -------> alterado (estava comentado)

	//INTCONbits.PEIE=1;		// Enable all peripheral interrupt sources
	//INTCONbits.GIE =1;      // Enable all interrupt sources
}

// Bloqueio do PIC em <val> ciclos de 100us (para fosc=40MHz)
//
void wait(unsigned int val)
{
    unsigned int i, j;

    for (i=0; i<val; ++i)
        for (j=0; j<140; ++j);
}

// Mquina de estados na troca de mensagens CAN
#define N_OPERATIONS    2           // Nmero de operaes numa transao
typedef enum {
    SEND_MESSAGE1 = 0,
    SEND_MESSAGE2 = 1
} enum_CanExchange_States;

static void processUsart(byte rx);

#pragma interrupt highISR save=section(".tmpdata")
void highISR(void)
{
    // Esperando por um caracter
    if (PIR1bits.RCIF)  {

        // Verificao da ocorrncia de um Framing Error (provocado por malformaes da palavra recebida)
        // e de Overrun Error (caracter recebido com o buffer de recepo (FIFO) cheio)
        if (RCSTAbits.FERR || RCSTAbits.OERR)  {
            RCREG;                                      // Remoo de um byte do buffer de recepo da USART
            RCSTAbits.CREN = 0;                         // Error cleared
            RCSTAbits.CREN = 1;
        }
        else  processUsart(RCREG);
    }
}

#pragma interruptlow lowISR save=section(".tmpdata")
void lowISR(void)
{
    // Variveis para a comunicao CAN
    bool res=true;                                          // Resultado da transmisso
    static byte scu2send=HOST_MCU+1;                        // SCU ID a enviar a mensagem CAN
    static enum_CanExchange_States can_op=SEND_MESSAGE1;    // Mquina de estados
/*
    if (INTCONbits.TMR0IE && INTCONbits.TMR0IF)  {

        usartCountTimeStart();               // Inicializao do valor inicial do Timer0
        PIN_TEST3=!PIN_TEST3;
    }
*/
    //_____ COMUNICAO CAN ____________________________________________________________
    // Envio de uma mensagem, via CAN, para um SCU para actualizao dos seus actuadores
    if (PIR1bits.TMR2IF)  {

        switch (can_op)  {
            // Envio da primeira mensagem de actualizao dos actuadores
            case SEND_MESSAGE1: res=canSendMsg(scu2send,0);
                                TXB1CONbits.TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
                                break;

            // Envio da segunda mensagem de actualizao dos actuadores
            case SEND_MESSAGE2: res=canSendMsg(scu2send,1);
                                TXB0CONbits.TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
                                break;
        }
        PIN_TEST2=res;

        if ((can_op=(can_op+1)%N_OPERATIONS)==0)  {
            scu2send=((scu2send-HOST_MCU)%N_SCU)+HOST_MCU+1;
            PIN_TEST=!PIN_TEST;
        }

        PIR1bits.TMR2IF=0;
    }
    //____________________________________________________________________________________

    // Receive Message 1
    if (PIR3bits.RXB0IF)  canReceiveMsg(0);

    // Receive Message 2
    if (PIR3bits.RXB1IF)  canReceiveMsg(1);
}


static void processUsart(byte rx)
{
    // Indicador se uma mensagem deve ser transmitida
    bool ready=false;

    // Se a receber uma stream de bytes e ocorreu timeout, limpar buffer de recepo
    // Este procedimento deve ser feito antes de avaliar o novo caracter recebido!
    if (usartStreamMode() && usartCountTimeout())
        usartResetRxBuff();
    
    // Em modo de recepo de um stream de bytes...
    if (usartStreamMode() || rx==MESSAGE_REQ || rx==MESSAGE_TEST)  {
    
        // Armazenamento do byte recebido no buffer de recepo
        usartStoreRx(rx,&ready);
    
        // Caso o buffer esteja ainda incompleto, reinicializar a contagem de tempo,
        // e esperar por mais um caracter
        if (!ready)  usartCountTimeStart();
    }
    
    // Se o buffer de recepo ainda no est cheio, voltar ao modo de recepo
    if (!ready)  return;
    
    // Aps este ponto, a transmisso da mensagem de resposta est em mente!
    
    // Processar mensagem de resposta
    usartProcessMsg();
    
    // Limpar buffer de recepo
    usartResetRxBuff();
    
    // Transmitir consecutivamente todos os bytes do buffer de transmisso
    // A flag TXIF  sempre inicializada a 0 sempre que uma escrita  efectuada no registo TXREG
    while (usartGetTx((byte *) &TXREG,NULL))  {
    
        // Inicializao da contagem de tempo
        usartCountTimeStart();
    
        // Esperar que o byte seja transmitido com sucesso
        while (!PIR1bits.TXIF && !usartCountTimeout());
        // Se o timeout foi atingido, forar sada do ciclo
        if (usartCountTimeout())  break;
    }
    
    // Reset dos buffer de transmisso e recepo
    usartResetTxBuff();
}
