// Esta verso retira o tratamento da USART atravs de interrupes e
// fornece-lhe a velocidade mxima, sendo executada a partir da funo
//  main. O CAN tem prioridade interrompendo esta rotina!

/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: MasterPrinc.c                                           |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "usart.h"

void usartMaster(void);

void main(void)
{	
    // Inicializao das estruturas globais
    initGlobal();

    //inicializacao das configuracoes do MCU
    initPic();
	
    // Comunicaes USART a tempo inteiro
    usartMaster();
}

void usartMaster(void)
{
    // Byte recebido pela USART
    byte rx;
    // Indicador se uma mensagem deve ser transmitida
    bool ready;

    // Loop infinito
    for (;;)  {

        // Sem mensagem de resposta
        ready=false;

        // Esperando por um caracter
        while (!PIR1bits.RCIF);
    
        // Verificao da ocorrncia de um Framing Error (provocado por malformaes da palavra recebida)
        // e de Overrun Error (caracter recebido com o buffer de recepo (FIFO) cheio)
        if (RCSTAbits.FERR || RCSTAbits.OERR)  {
            RCREG;                                      // Remoo de um byte do buffer de recepo da USART
            RCSTAbits.CREN = 0;                         // Error cleared
            RCSTAbits.CREN = 1;
            continue;
            //usartSendStatus(MESSAGE_ERROR);             // Preparar mensagem de erro
        }

        // Tudo OK
        // Leitura do byte recebido
        rx = RCREG;

        // Se a receber uma stream de bytes e ocorreu timeout, limpar buffer de recepo
        // Este procedimento deve ser feito antes de avaliar o novo caracter recebido!
        if (usartStreamMode() && usartCountTimeout())
            usartResetRxBuff();

        // Em modo de recepo de um stream de bytes...
        if (usartStreamMode() || rx==MESSAGE_REQ || rx==MESSAGE_TEST)  {

            // Armazenamento do byte recebido no buffer de recepo
            usartStoreRx(rx,&ready);

            // Caso o buffer esteja ainda incompleto, reinicializar a contagem de tempo,
            // e esperar por mais um caracter
            if (!ready)  usartCountTimeStart();
        }

        // Se o buffer de recepo ainda no est cheio, voltar ao modo de recepo
        if (!ready)  continue;

        // Aps este ponto, a transmisso da mensagem de resposta est em mente!

        // Processar mensagem de resposta
        usartProcessMsg();

        // Limpar buffer de recepo
        usartResetRxBuff();

        // Transmitir consecutivamente todos os bytes do buffer de transmisso
        // A flag TXIF  sempre inicializada a 0 sempre que uma escrita  efectuada no registo TXREG
        while (usartGetTx((byte *) &TXREG,NULL))  {
    
            // Inicializao da contagem de tempo
            usartCountTimeStart();
    
            // Esperar que o byte seja transmitido com sucesso
            while (!PIR1bits.TXIF && !usartCountTimeout());
            // Se o timeout foi atingido, forar sada do ciclo
            if (usartCountTimeout())  break;
        }
    
        // Reset dos buffer de transmisso e recepo
        usartResetTxBuff();
    }
}
