/*=======================================================================+
|             DEPARTAMENTO MECANICA UNIVERSIDADE AVEIRO                  |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              David Manuel Costa Gameiro          N:20281               |
|              Filipe Carvalho Mostardinha         N:22085               |
|                                                                        |
| Nome Ficheiro: InitPic.c                                               |
|                                                                        |
|                            SOURCE FILE                                 |
+=======================================================================*/


#include "pic.h"
#include "usart.h"
#include "can.h"

void isr(void);                             // Rotina de servio a interrupes de alta prioridade
void LOWisr(void);

// Quando ocorre um interrupt, o programa "salta" para esta linha do programa,
// que consequentemente reencaminha para a rotina responsvel pelo tratamento das interrupes.
#pragma code isrVector = 0x08
void isrVector(void)
{
_asm
	goto isr			                    // Salta para rotina de interrupt prioritario
_endasm
}

#pragma code

void initPic(void)
{
    /*------------------------------------------------------------------+
    |                    Definicoes dos TRIS do PIC                     |
    +------------------------------------------------------------------*/
    TRISA = 0b00000000;
    TRISB = 0b00000000;
    TRISC = 0b00001111;

    /*------------------------------------------------------------------+
    |                    Configurao do porto RA                       |
    +------------------------------------------------------------------*/

	ADCON1=0b00000110;      // All RA pins are digital

    /*------------------------------------------------------------------+
    |                    Inicializao de pinos                         |
    +------------------------------------------------------------------*/

    PIN_TEST=PIN_TEST2=PIN_TEST3=0;

    /*------------------------------------------------------------------+
    |          			 Inicializao das Comunicaes       		    |
    +------------------------------------------------------------------*/

    usartInit();            // Inicializao da USART
    canInit();              // Inicializao do CAN

    // Espera de 100 ms para garantir que todos os SCU esto
    // em plena operao!
//    wait(1000);

    /*------------------------------------------------------------------+
    |          			 Definies dos Interrupts             		    |
    +------------------------------------------------------------------*/

    // Activacao global das interrupcoes
	RCONbits.IPEN=0;		// Interrupt priority enable
	//INTCONbits.GIEH=1;		// Enable high priority interrupts		// -------> alterado (estava comentado)
	//INTCONbits.GIEL=1;      // Enable low priority interrupts		// -------> alterado (estava comentado)

	INTCONbits.PEIE=1;		// Enable all peripheral interrupt sources
	INTCONbits.GIE =1;      // Enable all interrupt sources
}

// Bloqueio do PIC em <val> ciclos de 100us (para fosc=40MHz)
//
void wait(unsigned int val)
{
    unsigned int i, j;

    for (i=0; i<val; ++i)
        for (j=0; j<140; ++j);
}

// Mquina de estados na troca de mensagens CAN
#define N_OPERATIONS    2           // Nmero de operaes numa transao
typedef enum {
    SEND_MESSAGE1 = 0,
    SEND_MESSAGE2 = 1
} enum_CanExchange_States;

#pragma interrupt isr save=section(".tmpdata")
void isr(void)
{
    // Variveis para a comunicao CAN
    bool res=true;                                          // Resultado da transmisso
    static byte scu2send=HOST_MCU+1;                        // SCU ID a enviar a mensagem CAN
    static enum_CanExchange_States can_op=SEND_MESSAGE1;    // Mquina de estados
/*
    if (INTCONbits.TMR0IE && INTCONbits.TMR0IF)  {

        usartCountTimeStart();               // Inicializao do valor inicial do Timer0
        PIN_TEST3=!PIN_TEST3;
    }
*/
    //_____ COMUNICAO CAN ____________________________________________________________
    // Envio de uma mensagem, via CAN, para um SCU para actualizao dos seus actuadores
    if (PIR1bits.TMR2IF)  {

        switch (can_op)  {
            // Envio da primeira mensagem de actualizao dos actuadores
            case SEND_MESSAGE1: TXB1CONbits.TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
                                res=canReceiveMsg(1);
                                canSendMsg(scu2send,0);
                                break;

            // Envio da segunda mensagem de actualizao dos actuadores
            case SEND_MESSAGE2: TXB0CONbits.TXREQ=0;                // Se a transmisso falhou, abortar retransmisses
                                res=canReceiveMsg(0);
                                canSendMsg(scu2send,1);
                                break;
        }
        PIN_TEST2=res;

        if ((can_op=(can_op+1)%N_OPERATIONS)==0)  {
            scu2send=((scu2send-HOST_MCU)%N_SCU)+HOST_MCU+1;
            PIN_TEST=!PIN_TEST;
        }

        PIR1bits.TMR2IF=0;
    }
    //____________________________________________________________________________________
}
