/*=======================================================================+
| DEPARTAMENTO MECANICA + DEPARTAMENTO DE ELECTRNICA E TELECOMUNICAES |
|                       DA UNIVERSIDADE AVEIRO                           |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              Milton Ruas da Silva                N:21824 (EET)         |
|              David Manuel Costa Gameiro          N:20281 (DEM)         |
|              Filipe Carvalho Mostardinha         N:22085 (DEM)         |
|                                                                        |
| Nome Ficheiro: candrives.c                                             |
| Drives forenecidos pela Microchip adaptados para performance em        |
| velocidade.                                                            |
|                                                                        |
|                                 C FILE                                 |
+=======================================================================*/


#include "CanDrivers.h"
#include <p18f258.h>


// Inicializao do mdulo CAN
//
void myCANInitialize(word mask, word filter1, word filter2)
{
    // ___ ENTRADA NO MODO DE CONFIGURAO ___
    
    // Request Disable Mode
    CANCON = (byte) (0b001 << 5);
    // Waiting for disabled mode to be active
    while ((CANSTAT >> 5) != 0b001);

    // Request Configuration Mode
    CANCON = (byte) (0b100 << 5);
    // Waiting for Configuration Mode to be active
    while ((CANSTAT >> 5) != 0b100);

    // ___ CONFIGURAO DO BIT RATE ___

    BRGCON1  = 0;
    BRGCON1 |= (byte) ((FASE_JUMP-1) << 6);     // Maximum Time Quantums for jump phase
    BRGCON1 |= (byte) BRP;                      // Baud rate

    BRGCON2  = 0;
    BRGCON2bits.SEG2PHTS = 1;                   // Phase Segment 2 freely programmable
    BRGCON2bits.SAM      = 0;                   // Bus Line is sampled once at the sample point
    BRGCON2 |= (byte) ((FASE_SEG1-1) << 3);     // Time Quantums for segment 1 phase
    BRGCON2 |= (byte) (FASE_PROP-1);            // Time Quantums for propagation phase

    BRGCON3  = 0;
    BRGCON3bits.WAKFIL = 0;                     // CAN bus line filter is not used for wake-up
    BRGCON3 |= (byte) (FASE_SEG2-1);            // Time Quantums for segment 2 phase

    // ___ CONFIGURAO DAS MSCARAS ___

    RXM1SIDH = (byte) (mask >> 3);              // Only one mask will be used
    RXM1SIDL = (byte) (mask << 5);
    RXM0SIDH = RXM1SIDH;
    RXM0SIDL = RXM1SIDL;

    // ___ CONFIGURAO DOS FILTROS ___

    // Filtro para o redireccionamento de mensagens para o buffer 0
    RXF0SIDH = RXF1SIDH = (byte) (filter1 >> 3);
    RXF0SIDL = RXF1SIDL = (byte) (filter1 << 5);

    // Filtro para o redireccionamento de mensagens para o buffer 1
    RXF2SIDH = RXF3SIDH = RXF4SIDH = RXF5SIDH = (byte) (filter2 >> 3);
    RXF2SIDL = RXF3SIDL = RXF4SIDL = RXF5SIDL = (byte) (filter2 << 5);

    // ___ CONFIGURAR ENTRADA ___

    // Buffer de recepo 0
    RXB0CONbits.RXM1  = 0;      // Receive only valid messages with standard identifier
    RXB0CONbits.RXM0  = 1;

    RXB0CONbits.RXRTRRO  = 0;   // No remote transfer request
    RXB0CONbits.RXB0DBEN = 0;   // No receive buffer0 overflow to receive Buffer1
    RXB0CONbits.JTOFF    = 0;   // Allows Jump Table offset between 1 and 0
    RXB0CONbits.FILHIT0  = 0;   // Acceptance Filter 0 (RXF0)

    PIR3bits.RXB0IF   = 0;      // Interrupt flag cleared
    RXB0CONbits.RXFUL = 0;      // Receive buffer is open to receive a new message
    
    // Buffer de recepo 1
    RXB1CONbits.RXM1  = 0;      // Receive only valid messages with standard identifier
    RXB1CONbits.RXM0  = 1;
    RXB1CONbits.RXRTRRO  = 0;   // No remote transfer request

    RXB1CONbits.FILHIT2  = 0;
    RXB1CONbits.FILHIT1  = 1;
    RXB1CONbits.FILHIT0  = 0;   // Acceptance Filter 2 (RXF2)

    PIR3bits.RXB1IF   = 0;      // Interrupt flag cleared
    RXB1CONbits.RXFUL = 0;      // Receive buffer is open to receive a new message
    
    // ___ CONFIGURAR SADA ___

    // Messages will be transmitted with standard identifier
    TXB0SIDLbits.EXIDE = 0;
    TXB1SIDLbits.EXIDE = 0;
    TXB2SIDLbits.EXIDE = 0;

    // Frame remote transmission not requested
    TXB0DLCbits.TXRTR = 0;
    TXB1DLCbits.TXRTR = 0;
    TXB2DLCbits.TXRTR = 0;

    // (Implicitily all status bits are cleared and transmission is allowed)
    TXB0CON = 0b11;             // Buffer 0 will have highest priority
    TXB1CON = 0b10;
    TXB2CON = 0b01;             // Buffer 2 will have lowest priority

    // ___ ENTRADA NO MODO NORMAL ___

    // Request Normal mode
    CANCON = (byte) (0b000 << 5);
    // Waiting for normal mode to be active
    while ((CANSTAT >> 5) != 0b000);
}

// _____ DEVICE DRIVERS DE TRANSMISSO DE MENSAGENS CAN __________________________________________

// Checking CAN status
static enum CAN_STATUS checkCanStatus(void);

// Envio de uma mensagem utilizando o primeiro buffer de transmisso vazio
//
bool myCANSendMessage(word id, byte* data, byte length, enum CAN_STATUS *status)
{
    if (!myCANSendMessage0(id,data,length,status))
        if (!myCANSendMessage1(id,data,length,status))
            if (!myCANSendMessage2(id,data,length,status))
                return false;

    return true;
}

// Envio de uma mensagem utilizando o buffer de transmisso 0
//
bool myCANSendMessage0(word id, byte* data, byte length, enum CAN_STATUS *status)
{
    byte i;
    byte window=CANCON & 0b00001110;

    // Checking CAN status
    *status=checkCanStatus();

    // Se o buffer de transmisso no est livre, abortar!
    if ( TXB0CONbits.TXREQ )   return false;

    // Clear the transmission interrupt flag.
    PIR3bits.TXB0IF = 0;

    // TxBuffer0 is empty.  Set WIN bits to point to TXB0
    CANCON &= 0b11110001;
    CANCON |= (0b100 << 1);

    // Identifier
    TXB0SIDH = (byte) (id >> 3);
    TXB0SIDL &= 0b00011111;
    TXB0SIDL |= (byte) (id << 5);
    // Message will be transmitted with standard identifier
    TXB0SIDLbits.EXIDE = 0;

    // Message data length
    TXB0DLC &= 0b11110000;
    TXB0DLC |= (length & 0b1111);
    // Frame remote transmission not requested
    TXB0DLCbits.TXRTR = 0;

    // Data to be sent
    for (i=0; i<length; ++i)  *(&TXB0D0+i)=data[i];

    // Highest priority in transmission
    TXB0CONbits.TXPRI1=1;
    TXB0CONbits.TXPRI0=1;

    // Send message
    TXB0CONbits.TXREQ = 1;

    // Reposio da janela de endereamento
	CANCON &= 0b11110001;
    CANCON |= window;

    return true;
}

// Envio de uma mensagem utilizando o buffer de transmisso 1
//
bool myCANSendMessage1(word id, byte* data, byte length, enum CAN_STATUS *status)
{
    byte i;
    byte window=CANCON & 0b00001110;

    // Checking CAN status
    *status=checkCanStatus();

    // Se o buffer de transmiss
    if ( TXB1CONbits.TXREQ )  return false;

    // Clear the transmission interrupt flag.
    PIR3bits.TXB1IF = 0;

    // TxBuffer1 is empty.  Set WIN bits to point to TXB1
    CANCON &= 0b11110001;
    CANCON |= (0b011 << 1);

    // Identifier
    TXB1SIDH = (byte) (id >> 3);
    TXB1SIDL &= 0b00011111;
    TXB1SIDL |= (byte) (id << 5);
    // Message will be transmitted with standard identifier
    TXB1SIDLbits.EXIDE = 0;

    // Message data length
    TXB1DLC &= 0b11110000;
    TXB1DLC |= (length & 0b1111);
    // Frame remote transmission not requested
    TXB1DLCbits.TXRTR = 0;

    // Data to be sent
    for (i=0; i<length; ++i)  *(&TXB1D0+i)=data[i];

    // Priority in transmission
    TXB0CONbits.TXPRI1=1;
    TXB0CONbits.TXPRI0=0;

    // Send message
    TXB1CONbits.TXREQ = 1;

    // Reposio da janela de endereamento
	CANCON &= 0b11110001;
    CANCON |= window;

    return true;
}

// Envio de uma mensagem utilizando o buffer de transmisso 2
//
bool myCANSendMessage2(word id, byte* data, byte length, enum CAN_STATUS *status)
{
    byte i;
    byte window=CANCON & 0b00001110;

    // Checking CAN status
    *status=checkCanStatus();

    // Se o buffer de transmiss
    if ( TXB2CONbits.TXREQ )  return false;

    // Clear the transmission interrupt flag.
    PIR3bits.TXB2IF = 0;

    // TxBuffer1 is empty.  Set WIN bits to point to TXB2
    CANCON &= 0b11110001;
    CANCON |= (0b010 << 1);

    // Identifier
    TXB2SIDH = (byte) (id >> 3);
    TXB2SIDL &= 0b00011111;
    TXB2SIDL |= (byte) (id << 5);
    // Message will be transmitted with standard identifier
    TXB2SIDLbits.EXIDE = 0;

    // Message data length
    TXB2DLC &= 0b11110000;
    TXB2DLC |= (length & 0b1111);
    // Frame remote transmission not requested
    TXB2DLCbits.TXRTR = 0;

    // Data to be sent
    for (i=0; i<length; ++i)  *(&TXB2D0+i)=data[i];

    // Lowest priority in transmission
    TXB0CONbits.TXPRI1=0;
    TXB0CONbits.TXPRI0=1;

    // Send message
    TXB2CONbits.TXREQ = 1;

    // Reposio da janela de endereamento
	CANCON &= 0b11110001;
    CANCON |= window;

    return true;
}


// _____ DEVICE DRIVERS DE RECEPO DE MENSAGENS CAN _____________________________________________

// Recepo de uma mensagem utilizando o primeiro buffer de recepo vazio
//
bool myCANReceiveMessage(word *id, byte *data, byte *length, enum CAN_STATUS *status)
{
    if (!myCANReceiveMessage0(id,data,length,status))
        if (!myCANReceiveMessage1(id,data,length,status))
            return false;

    return true;
}

// Recepo de uma mensagem utilizando o buffer de recepo 0
//
bool myCANReceiveMessage0(word *id, byte *data, byte *length, enum CAN_STATUS *status)
{
    byte i;
    byte window=CANCON & 0b00001110;        // Salvaguarda da janela de endereamento

    // Checking CAN status
    *status=checkCanStatus();

    // Verificao de um erro de mensagem invlida
    if (PIR3bits.IRXIF)  {
        PIR3bits.IRXIF = 0;
        *status |= CAN_STATUS_RXINVALIDMSG;
    }

    // Buffer de recepo cheio?
    if ( !RXB0CONbits.RXFUL )  return false;

    // Clear the reception interrupt flag.
    PIR3bits.RXB0IF = 0;

    // RXBuffer0 is full
    CANCON &= 0b11110001;
    CANCON |= (0b110 << 1);

    // Overflow check
    if (COMSTATbits.RXB0OVFL)  {
        // Overflow state cleared
        COMSTATbits.RXB0OVFL = 0;
        *status |= CAN_STATUS_RXOVERFLOW;
    }

    // Message Identifier
    *id  = ((word) RXB0SIDH << 3) | (RXB0SIDL >> 5);

    // Data length
    *length = RXB0DLC & 0b1111;

    // Message Data
    for (i=0; i<(RXB0DLC & 0b1111); ++i)  data[i]=*(&RXB0D0+i);

    // Buffer free
    RXB0CONbits.RXFUL=0;

    // Reposio da janela de endereamento
	CANCON &= 0b11110001;
    CANCON |= window;

    return true;
}

// Recepo de uma mensagem utilizando o buffer de recepo 1
//
bool myCANReceiveMessage1(word *id, byte *data, byte *length, enum CAN_STATUS *status)
{
    byte i;
    byte window=CANCON & 0b00001110;        // Salvaguarda da janela de endereamento

    // Checking CAN status
    *status=checkCanStatus();

    // Verificao de um erro de mensagem invlida
    if (PIR3bits.IRXIF)  {
        PIR3bits.IRXIF = 0;
        *status |= CAN_STATUS_RXINVALIDMSG;
    }

    // Buffer 1 cheio?
    if ( !RXB1CONbits.RXFUL )  return false;

    // Clear the reception interrupt flag.
    PIR3bits.RXB1IF = 0;

    // RXBuffer0 is full
    CANCON &= 0b11110001;
    CANCON |= (0b101 << 1);

    // Overflow check
    if (COMSTATbits.RXB1OVFL)  {
        // Overflow state cleared
        COMSTATbits.RXB1OVFL = 0;
        *status |= CAN_STATUS_RXOVERFLOW;
    }

    // Message Identifier
    *id  = ((word) RXB1SIDH << 3) | (RXB1SIDL >> 5);

    // Data length
    *length = RXB1DLC & 0b1111;

    // Message Data
    for (i=0; i<(RXB1DLC & 0b1111); ++i)  data[i]=*(&RXB1D0+i);

    // Buffer free
    RXB1CONbits.RXFUL=0;

    // Reposio da janela de endereamento
	CANCON &= 0b11110001;
    CANCON |= window;

    return true;
}

// Checking CAN status
//
static enum CAN_STATUS checkCanStatus(void)
{
    enum CAN_STATUS status=0;

    // Erro no barramento
    if (PIR3bits.ERRIF)  {
        PIR3bits.IRXIF=0;
        status |= CAN_STATUS_ERROR;
    }

    if (COMSTATbits.RXBP)
        status |= CAN_STATUS_RXPASSIVE;
    
    if (COMSTATbits.RXWARN)
        status |= CAN_STATUS_RXWARNING;

    if (COMSTATbits.TXBP)
        status |= CAN_STATUS_TXPASSIVE;

    if (COMSTATbits.TXWARN)
        status |= CAN_STATUS_TXWARNING;

    if (COMSTATbits.TXBO)
        status |= CAN_STATUS_BUSOFF;

    return status;
}
