/*=======================================================================+
| DEPARTAMENTO MECANICA + DEPARTAMENTO DE ELECTRNICA E TELECOMUNICAES |
|                       DA UNIVERSIDADE AVEIRO                           |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              Milton Ruas da Silva                N:21824 (EET)         |
|              David Manuel Costa Gameiro          N:20281 (DEM)         |
|              Filipe Carvalho Mostardinha         N:22085 (DEM)         |
|                                                                        |
| Nome Ficheiro: global.h                                                |
|                                                                        |
|                            HEADER FILE                                 |
+=======================================================================*/

#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <p18f258.h>
#include "types.h"

// _____ ESTADO DE FUNCIONAMENTO ESTTICO DO SISTEMA __________________________________________________________________

// Endereos
#define HOST_MCU    0                   // ID da unidade Master
#define HOST_SCU   (PORTC & 0b1111)     // ID deste SCU (Valor presente no PORTC)

// Dados sobre o SCU
#define N_SCU             8                                 // Nmero de Slave Controller Units
#define N_SERVOS          3                                 // Nmero de servos por SCU
#define N_SPECIAL_SENSORS 4                                 // Nmero de sensores de fora por p


// _____ ESTRUTURAS GLOBAIS DO SISTEMA ________________________________________________________________________________

// Tipo enumerado do gnero de controlador implementado em cada junta
typedef enum {
    NO_CONTROL  = 0b00,              // Funcionamento em malha aberta
    COP_CONTROL = 0b01,              // Controlo de equilbrio no tronco
    INC_CONTROL = 0b10,              // Controlo das foras de reaco nos ps
    GIRO_CONTROL= 0b11               // Controlo de posio e velocidade
} enum_controlType;

// Estrutura descritiva do estado de um servo
typedef struct {
    signed char position;                                   // Posio
    signed char velocity;                                   // Velocidade
    unsigned char current;                                  // Corrente consumida
} struct_servo;

// Estrutura descritiva dos sensores
typedef struct {
    struct {
        bool pwm;                                        // Motores ligados/desligados
        bool calib;                                      // Calibrao ligada/desligada
        bool deadlineError;                              // Violao de deadline
        bool motionFinAll;                               // Todos os motores terminaram o movimento
        bool motionFinOne;                               // Um dos motores terminaram o movimento
        bool motionFin[N_SERVOS];                        // Movimento terminado
    } sysStatus;                                    // Estado sensorial do sistema
    struct_servo servo[N_SERVOS];                   // Sensores dos servos (posio, velocidade e corrente)
    unsigned char special[N_SPECIAL_SENSORS];       // Sensores especiais
} struct_sensors;

// Estrutura descritiva dos actuadores
typedef struct {
    // Estrutura de actuao de status para cada SCU
    struct {
        bool pwm;                                       // PWM on/off
        bool calib;                                     // Dynamic calibration on/off
    } sysStatus;                                    // Estado da actuao do sistema
    struct {
        byte kp, kd, ki;                                    // Parmetros Kp, Kd e Ki
        byte type;                                          // Tipo de Controlador
    } control[N_SERVOS];                            // Controlador
    struct_servo servo[N_SERVOS];                   // Informao de actuao (posio, velocidade e corrente)
} struct_actuators;

// Variveis globais descritivas do sistema
extern volatile struct_sensors   sensors;                   // Sensores
extern volatile struct_actuators actuators;                 // Actuadores

// _____ FUNES DE INICIALIZAO DO SISTEMA __________________________________________________________________________

// Inicializao de todas as estruturas globais
extern void initGlobal(void);

#endif
