/*=======================================================================+
| DEPARTAMENTO MECANICA + DEPARTAMENTO DE ELECTRNICA E TELECOMUNICAES |
|                       DA UNIVERSIDADE AVEIRO                           |
|               Projecto Humanoide (PHUA - MODULROB)                     |
|                                                                        |
| Programadores:                                                         |
|              Milton Ruas da Silva                N:21824 (EET)         |
|              David Manuel Costa Gameiro          N:20281 (DEM)         |
|              Filipe Carvalho Mostardinha         N:22085 (DEM)         |
|                                                                        |
| Nome Ficheiro: pic2.c                                                  |
|                                                                        |
|                               C FILE                                   |
+=======================================================================*/

#include "pic2.h"
#include "can.h"
#include "mymaths.h"
#include <string.h>

// ______ DEFINIO DAS FUNES DE SERVIO A INTERRUPES ___________________________________
void highISR(void);						// Prottipo da funo de servio a interrupes
void lowISR(void);

// Interrupt de alta prioridade
#pragma code InterruptVectorHigh = 0x08	//Quando ocorre um interrupt o programa "salta" para este endereo
void InterruptVectorHigh (void)
{
    _asm  goto highISR  _endasm
}

// Interrupt de baixa prioridade
#pragma code InterruptVectorLow = 0x18	//Quando ocorre um interrupt o programa "salta" para este endereo
void InterruptVectorLow (void)
{
    _asm  goto lowISR  _endasm
}
#pragma code


// ==========================================================================================
// Device Drivers
// ==========================================================================================

// Inicializacoes relativas ao microcontrolador
//
void initPic(void)
{
    byte i;

    // Inicializao das estruturas globais
    initGlobal();

    // Programao dos portos de entrada/sada
	TRISA=0b00000001;		// RA0 input / RA1..RA7 outputs
	TRISB=0b00000000;		// RB0...RB7 outputs
	TRISC=0b00001111;		// RC0...RC4 outputs; RC3..RC0 inputs

    // Programacao da ADC

	ADCON0=0b10000001;      // Fosc/64, channel 0(RA0), enable ADC converter
	ADCON1=0b01001110;      // Left justified; RA0 analog; the rest are digital

	IPR1bits.ADIP=1;		// Low Priority interrupt
	PIE1bits.ADIE=0;		// Low Priority interrupt

	// Inicializao das comunicaes CAN
	initCan();
    // Interrupes (recepo) de baixa prioridade
    IPR3bits.RXB0IP=IPR3bits.RXB1IP=0;
    // Activa os interrupts dos buffers recepcao CAN
    PIE3bits.RXB0IE=PIE3bits.RXB1IE=1;  // Interrupts Enable

    initUsart();

    // Activacao global das interrupcoes
	RCONbits.IPEN=1;		// Interrupt priority enable
	INTCONbits.GIEH=1;		// Enable high priority interrupts
	INTCONbits.GIEL=1;      // Enable low priority interrupts

    // Esperar pela primeira mensagem CAN
    while (!can_message_received);
}


// ==========================================================================================
// Funes relativas a interrupes
// ==========================================================================================


// Rotina de servio s interrupes de alta prioridade
//
// O pragma seguinte indica que a rotina definida atende interrupes de alta prioridade,
// com uma seco de dados prpria com o nome isr_tmp, e que no  necessrio guardar o
// contexto da seco de dados geral.
#pragma interrupt highISR nosave=section(".tmpdata")
void highISR(void)
{
}

// Rotina de atendimento s interrupes de baixa prioridade
#pragma interruptlow lowISR save=section(".tmpdata")
void lowISR(void)
{
    bool pwm=0;

    if (PIR3bits.RXB0IF)  {

        TESTPIN3=1;

        // Se a transmisso da segunda mensagem falhou, abortar retransmisses
        TXB1CONbits.TXREQ=0;

        // Recepo de mensagem CAN
        receiveCanMessage(0);

        // Transmitir mensagem de sada
        // S a transmite se antes e depois da recepo, o PWM estava em low
        if (!pwm)  sendCanMessage(0);

        TESTPIN3=0;
    }

    if (PIR3bits.RXB1IF)  {

        TESTPIN3=1;

        TXB0CONbits.TXREQ=0;                // Se a transmisso falhou, abortar retransmisses

        // Recepo de mensagem CAN
        receiveCanMessage(1);

        // Transmitir mensagem de sada
        // S a transmite se antes e depois da recepo, o PWM estava em low
        if (!pwm)  sendCanMessage(1);

        TESTPIN3=0;
    }
}
