#include <p18f258.h>

#pragma config OSC  = HSPLL
#pragma config PWRT = ON
#pragma config BOR  = ON
#pragma config BORV = 27
#pragma config WDT  = OFF
#pragma config STVR = ON
#pragma config LVP  = OFF

#define PIN_TEST    PORTCbits.RC4

// ****************************************************************************
// Configurao do Timer2 para transmisso peridica de mensagens CAN
#define F_CPU               10
#define TIMER2_CONFIG       0b00001111
#define TIMER2_PRES_CODE    0b10
#define TIMER2_POST_CODE    0b0001
#define TIMER2_PRES 16
#define TIMER2_POST 2
#define TIMER2_OUT  500         // Periodo em microsegundos
#define TIMER2_PER  (unsigned char) ((unsigned int) TIMER2_OUT*F_CPU/((unsigned int) TIMER2_PRES*TIMER2_POST)-1)

void main(void)
{
    /*------------------------------------------------------------------+
    |                    Definicoes dos TRIS do PIC                     |
    +------------------------------------------------------------------*/
    TRISA = 0b00000000;
    TRISB = 0b00000000;
    TRISC = 0b00000000;

    /*------------------------------------------------------------------+
    |                    Configurao do porto RA                       |
    +------------------------------------------------------------------*/

	ADCON1=0b00000110;      // All RA pins are digital

    /*------------------------------------------------------------------+
    |                    Inicializao de pinos                         |
    +------------------------------------------------------------------*/

    PIN_TEST=0;

    /*------------------------------------------------------------------+
    |                    Inicializao de timers                        |
    +------------------------------------------------------------------*/

    // Configurao do Timer2 para gerao de interrupes de 1000 em 1000 us
    // Este interrupt ser usado para transmisses de mensagens CAN
/*    T2CONbits.TOUTPS3=(TIMER2_POST_CODE>>3) & 1;
    T2CONbits.TOUTPS2=(TIMER2_POST_CODE>>2) & 1;
    T2CONbits.TOUTPS1=(TIMER2_POST_CODE>>1) & 1;
    T2CONbits.TOUTPS0=TIMER2_POST_CODE & 1;                 // Postscaler

    T2CONbits.T2CKPS1=(TIMER2_PRES_CODE >> 1) & 1;          // Prescaler
    T2CONbits.T2CKPS0=TIMER2_PRES_CODE & 1;
*/
    T2CON=TIMER2_CONFIG;

    PR2=TIMER2_PER;             // Definio do perodo
    TMR2=0;                     // Inicializao do contador
    T2CONbits.TMR2ON=1;         // Timer2 ON
    IPR1bits.TMR2IP=0;          // Interrupo de alta prioridade
    PIE1bits.TMR2IE=1;          // Activao das interrupes
    PIR1bits.TMR2IF=0;          // Activao das interrupes

    /*------------------------------------------------------------------+
    |          			 Definies dos Interrupts             		    |
    +------------------------------------------------------------------*/

    // Activacao global das interrupcoes
	RCONbits.IPEN=1;		// Interrupt priority enable
	INTCONbits.GIEH=1;		// Enable high priority interrupts		// -------> alterado (estava comentado)
	INTCONbits.GIEL=1;      // Enable low priority interrupts		// -------> alterado (estava comentado)

	INTCONbits.PEIE=1;		// Enable all peripheral interrupt sources
	INTCONbits.GIE =1;      // Enable all interrupt sources

    PIN_TEST=1;

    for (;;);
}


void isr(void);                             // Rotina de servio a interrupes de alta prioridade
void lowisr(void);

// Quando ocorre um interrupt, o programa "salta" para esta linha do programa,
// que consequentemente reencaminha para a rotina responsvel pelo tratamento das interrupes.
#pragma code isrVector = 0x08
void isrVector(void)
{
_asm
goto isr			                    // Salta para rotina de interrupt prioritario
_endasm
}
#pragma code

#pragma interrupt isr
void isr(void)
{
}


#pragma code lowisrVector = 0x18
void lowisrVector(void)
{
_asm
goto lowisr			                    // Salta para rotina de interrupt prioritario
_endasm
}
#pragma code

#pragma interruptlow lowisr
void lowisr(void)
{
    if (PIR1bits.TMR2IF)  {

        PIN_TEST=!PIN_TEST;

        PIR1bits.TMR2IF=0;
    }
}
